;;; This file is an addition to the GNU Emacs. GNU Emacs and the herein GOLD
;;; additions are distributed in the hope that it will be useful, but without
;;; ANY WARRANTY.  No author or distributor accepts responsibility to anyone
;;; for the consequences of using it or for whether it serves any particular
;;; purpose or works at all, unless he says so in writing.
;;; Everyone is granted permission to copy, modify and redistribute GNU Emacs,
;;; but only under the conditions described in the GNU Emacs General Public Li-
;;; cense.   A copy of this license is supposed to have been given to you along
;;; with GNU Emacs so you can know your rights and responsibilities.  It should
;;; be in a file named COPYING.  Among other things, the copyright notice and
;;; this notice must be preserved on all copies.
;;; Refer to the GNU Emacs General Public License for full details.
;;;
;;;############################################################################
;;;###  Database like input/search/query functions for ASCII-Address-Files  ###
;;;###  This file is included (direct- or auto-loaded) by the main entry    ###
;;;###  file >>> gold.el <<< => Refer to that file for more informations.   ###
;;;############################################################################
;;; Bug reports and suggestions are always welcome !
;;
(defun adr-mode ()
  "Modified text-mode with database like functions for ASCII-Adress-Files."
  (interactive)
  (kill-all-local-variables)
  (text-mode) (setq overwrite-mode t)
  (use-local-map adr-mode-map)
  (setq context-help-function   'help-adr-field)
  (setq context-update-function 'update-day-month-year)
  (setq mode-name "Address-DB") (setq major-mode 'adr-mode)
  (run-hooks 'adr-mode-hook))

(defvar adr-mode-map nil "Local keymap used in adr-mode mode.")
(setq adr-mode-map nil)
(if     adr-mode-map () (setq adr-mode-map (make-sparse-keymap))
  (define-key adr-mode-map "\t"         'next-DB-field)
  (define-key adr-mode-map [prior]      'previous-DB-entry)
  (define-key adr-mode-map [next]       'next-DB-entry)
  (define-key adr-mode-map [f28]        'new-DB-entry)
  (define-GOLD-local adr-mode-map [prior] 'bottom-of-DB-entry)
  (define-GOLD-local adr-mode-map [next]  'top-of-DB-entry)
  (define-GOLD-local adr-mode-map [f14]   'new-DB-entry)
)

(defvar DB-field-pattern ": \\|/ \\|^ *.) "
                         "RE-Search pattern for start of DB input fields.")
(defun next-DB-field () "Jump to next input field of ASCII-File Database."
  (interactive)
  (if advance (re-search-forward  DB-field-pattern)
              (backward-char 2) (re-search-backward DB-field-pattern)
              (search-forward " ")))

(defun next-previous-DB-entry () "Jump to top of following/preceeding DB-Entry."
  (interactive) (if advance (next-DB-entry) (previous-DB-entry)))
(defun previous-DB-entry () "Jump to top of preceeding DB-Entry." (interactive)
   (search-backward "@DB-Separator-Line>>>" nil t)
   (if (search-backward "@DB-Separator-Line>>>" nil t)
       (progn (search-forward ": ") (line-to-top-of-window))
       (move-to-beginning) (search-forward ": ")
       (message "First DB-Entry in file !")))
(defun next-DB-entry () "Jump to top of next following DB-Entry." (interactive)
  (if (search-forward "@DB-Separator-Line>>>" nil t)
      (progn (re-search-forward DB-field-pattern) (line-to-top-of-window))
      (message "Last DB-Entry in file !")))
;
(defun top-of-DB-entry () "Jump to top input field of current DB-Entry."
  (interactive) (re-search-backward "@DB-Separator-Line>>>\\|\\`" nil t)
                (re-search-forward  DB-field-pattern nil t)
                (line-to-top-of-window))
(defun end-of-DB-entry () "Jump beyond last input field of current DB-Entry."
  (interactive) (re-search-forward "@DB-Separator-Line>>>\\|\\'" nil t)
                (beginning-of-line) (re-search-backward "[^ \n]")
                (forward-char 1) (line-to-bottom-of-window))
;
(defun new-DB-entry () " "
   (interactive)
   (if (search-forward "@DB-Separator-Line>>>" nil t) (beginning-of-line)
       (move-to-end) (insert "\n"))
   (insert "@DB-Separator-Line>>>\n")
   (insert "Firma:                              Bereich: \n")
   (insert "                                             \n")
   (insert "                                    Produkt: \n")
   (insert "                                             \n")
   (insert "                                    Groesse: \n") ;; Prioritaet ?
   (insert "        \n")
   (insert "   Ansprechpartner                           Abt.   Vorwahl  Telefon   Fax\n")
   (insert "1)                                         :       :       /    -      -  \n\n")
   (insert "Programm:                          Spektrum: \n")
   (insert "Pre/Post:                                    \n")
   (insert "Rechner :                          INTES-Auftragsberechnung: \n\n")
   (insert "Quellen : \n")
   (insert "Kontakte:                                            Status: 0\n")
   (insert "NextStep: ??-Feb-94 Anruf\n\n")
   (insert "Freier Textbereich:\n\n")
   (search-backward "Firma: ") (line-to-top-of-window) (search-forward ": "))
;

;;***************************************************
;;*****  Context-Help on DB input fields !  *********
;;***************************************************
(defun help-adr-field () "Show help for actual Address-DB input field."
  (interactive)
  (let*((key (DB-field-at-point))
        (actual-buffer (current-buffer))
        (help-text (cdr (assoc key adr-help-alist))))
       (if (equal (substring key 0 1) " ") (plz);;; Convert 4PLZ->5PLZ !!!!
       (if help-text (with-output-to-temp-buffer "*Help*"
                     (princ help-text) (print-help-return-message))
              (beep) (message (concat "No such DB field \""
                                       key "\" (search failed)")))
       (set-buffer actual-buffer))))

(defun DB-field-at-point ()
   "Returns the name specifier of the actual DB-Field at point."
   (let ((oldpos (point))
         (pattern (mapconcat 'car adr-help-alist "\\|")))
        (re-search-forward " [^ ]\\|$" nil t)
        (prog1 (if (re-search-backward pattern nil t)
                   (buffer-substring (match-beginning 0) (match-end 0))
                   (eval "???"))
               (goto-char oldpos))))
;;***************************************************************************
(defvar adr-help-alist nil "A List of Address-DB fields with Help-Texts")
(setq adr-help-alist '(;;; A List of Address-DB input fields with Help-Texts
;;******
("Firma:" .
"Firmenanschrift mit Strasse, Postfach, PLZ und Stadt (evtl. Staat), z.B.:

Firma: Willhelm @Tell AG            Firma: INTES GmbH
       Hohle Gasse 14                      Industriestrasse -2
       Postleitzahl 1234
    CH-5623 Luzern                       D-7000 Stuttgart-80
       Schweiz

* Der Namen des Ansprechpartners und seiner Abteilung erscheinen hier NICHT !
* Falls keine Postleitzahl bzw. Strasse bekannt ist bleibt das entsprechende
  Feld einfach leer (PLZ und Stadt immer auf 4.Zeile).
* Soll als Haupt-Namensschluessel ein anderes als das 1.Wort des Firmennamens
  verwendet werden, so ist dieses durch ein vorangestelltes @ zu kennzeichnen.")
;;********
("^ [{\bf ]*\\([DWO][-]\\)?[1-9][0-9][0-9][0-9][^0-9]" .
"Line with old Postleitzahl: Convert 4PLZ->5PLZ !")
;;********
("Bereich:"  .
"Bereich:
Optionale Bezeichnung von Werk, Bereich oder Abteilung (bzw. auch Person)
als Abgrenzung gegenueber anderen/unabhaengigen Kontakten in derselben Firma !")
;;********
("Produkt:"  .
"Produkt: Liste von Firmenprodukten bzw. -entwicklungen.")
;;********
("Groesse:"  .
"Groesse: Firmengroesse als Bezeichnung (Klein, Mittel, Gross oder Konzern)
         und/oder ungefaehre Zahl der Mitarbeiter (falls bekannt).")
;;*********
("  Ansprechpartner  " . "Liste mit Namen, Abteilung, Telefon- und Fax-Nr.")
("  Abt.  "            . "Abteilungsbezeichnung, wie sie in der Anschrift erscheinen soll.")
("  Vorwahl  "         . "Ortsvorwahl bzw. gemeinsame Vorwahl von Tel+Fax.")
("  Telefon  "         . "Telefonnummer (direkte Durchwahl, falls moeglich).")
("  Fax  "             . "Von Telefondurchwahl abweichende Faxnummer.")
("1) " . "Name, Abteilung, Telefon- und Fax-Nr. des 1.Ansprechpartners.")
("2) " . "Name, Abteilung, Telefon- und Fax-Nr. des 2.Ansprechpartners.")
("3) " . "Name, Abteilung, Telefon- und Fax-Nr. des 3.Ansprechpartners.")
("4) " . "Name, Abteilung, Telefon- und Fax-Nr. des 4.Ansprechpartners.")
("5) " . "Name, Abteilung, Telefon- und Fax-Nr. des 5.Ansprechpartners.")
;;*********
("Programm:" .
"Derzeit genutzte FE-Programm(e).
Z.B.: MSC/NASTRAN, ANSYS, TPS10, ...")
;;******
("Pre/Post:" .
"Derzeit genutztes Pre/Post-Programm(e).
Z.B.: PATRAN, I-DEAS/Supertab, Medina/Disco, FEM-Gen/View, ...")
;;******
("Rechner :"  .
"Derzeit (und/oder in naechster Zukunft) fuer FE genutzte Rechnertypen.")
;;******
("Spektrum:" .
"Liste von typischen FE-Anwendungen
(sowohl Massen- als auch Spezialanwednungen).")
;;*******************
("INTES-Auftragsberechnung:" .
"Ja/Nein/Vielleicht-Status zu Berechnungsauftraegen an INTES-Engineering")
;;******
("Quellen :" .
"Angabe der Adressquelle(n), z.B.: Bezeichnung con VDI-Katalog, Tagung,
Datenbasis o.ae. von der die Adress-Information entnommen wurde.")
;;******
("Kontakte:" .
"Historie der Kontakte (mit Datum/Namenskuerzel/Kurztext und Kunden-Status.
Das aktuelle Datum kann mit Gold-F15 eingefuegt bzw. upgedated werden
(letzteres, falls schon ein Datum auf der aktueller Zeile existiert).")
;;******
("Status:" .
"     Kunden-Status fuer INTES Adressdatei:
     -------======------------------------
-3 - Kein Interesse an FE (derzeit)         4 - Besuch/Demo vereinbart
-2 - Kein Interesse an PERMAS (derzeit)     5 - Benchmark vereinbart
-1 - Noch kein Kontakt, Interesse fraglich  6 - Benchmark erledigt
 0 - Noch kein Kontakt                      7 - Testinstallation
 1 - Noch kein Kontakt, Qualifiziert        8 - Vertragsverhandlungen laufen
 2 - Telefonischer Konakt mit Interesse     9 - Vertrag vor Unterschrift
 3 - Info verschickt, nachhaken            10 - Kunde (jetzt gibts Sekt)")
;;********
("NextStep:" .
"Datum und Art bzw. Bezeichnung der naechsten INTES-Schritte.
Dieses Feld wird in Zukunft fuer automatische Termin-Tools (aehnlich calendar)
Verwendung finden.
Das aktuelle Datum kann mit Gold-F15 eingefuegt bzw. upgedated werden
(letzteres, falls schon ein Datum auf der aktueller Zeile existiert).")
;;********
("Freier Textbereich:" . "Frei editierbarer Bereich fuer ergaenzende Texte.")
;;********
("bereich:" . "Frei editierbarer Bereich fuer ergaenzende Texte.")
;;*********
))

;;----------------------------------------------------------------------------
(provide 'gold_adr)
