;;; This file is an addition to the GNU Emacs. GNU Emacs and the herein GOLD
;;; additions are distributed in the hope that it will be useful, but without
;;; ANY WARRANTY.  No author or distributor accepts responsibility to anyone
;;; for the consequences of using it or for whether it serves any particular
;;; purpose or works at all, unless he says so in writing.
;;; Everyone is granted permission to copy, modify and redistribute GNU Emacs,
;;; but only under the conditions described in the GNU Emacs General Public Li-
;;; cense.   A copy of this license is supposed to have been given to you along
;;; with GNU Emacs so you can know your rights and responsibilities.  It should
;;; be in a file named COPYING.  Among other things, the copyright notice and
;;; this notice must be preserved on all copies.
;;; Refer to the GNU Emacs General Public License for full details.
;;;
;;;############################################################################
;;;###  Additional/different calendar & diary functions for gold.el !       ###
;;;###  This file is included (direct- or auto-loaded) by the main entry    ###
;;;###  file >>> gold.el <<< => Refer to that file for more informations.   ###
;;;############################################################################
;;; Bug reports and suggestions are always welcome !
;;
(gold-debug "gold_calendar")
;;--------------------------
(setq diary-file "~/.diary")          ;; Default diary-file
(setq diary-list-include-blanks t)
(setq diary-display-function 'diary-fancy-display); Default since Emacs 23.2
(add-hook 'diary-mark-entries-hook 'diary-mark-included-diary-files)
(add-hook 'diary-list-entries-hook 'diary-include-other-diary-files)
(add-hook 'diary-list-entries-hook 'diary-sort-entries t); last in diary-list
(add-hook 'calendar-today-visible-hook 'calendar-mark-today t)
;;
(setq calendar-date-style 'european) ;; DAY/MONTH (instead of month/day) DD-MMM-YY
(setq calendar-view-diary-initially-flag t)
(setq diary-number-of-entries [14 16 15 14 14 16 15])
;;
(setq diary-european-date-forms '((day "[/- ]" month "[^/0-9]")
                                  (day "[/- ]" month "[/- ]" year "[^0-9]")
                                  (backup day "[ -/]*" monthname "\\W+\\<[^*0-9]")
                                  (day "[ -/]*" monthname "[ -/]*" year "[^0-9]")
                                  (dayname "\\W")))
(setq diary-date-forms diary-european-date-forms)
;;

; User format 24:00 instead of 12:00pm -> crashes in emacs 23.3?
(setq calendar-time-display-form
    '(24-hours ":" minutes (if time-zone " (") time-zone (if time-zone ")")))


;; Appointments:
;;---------------
;(add-hook 'diary-hook 'appt-make-list-once)
;(defun appt-make-list-once () "Call appt-make-list only once per session !"
;  (if (boundp 'appt-time-msg-list) (if appt-time-msg-list nil (appt-make-list))
;                                   (appt-make-list)))
(defun termin () "Add an appointment" (interactive)
      (if (not (boundp 'appt-time-msg-list)) (diary))
      (call-interactively 'appt-add))
(setq issue-appointments-message t)

;; Holidays:
;;-----------
(defun holiday-german-easter-etc ()
  "List of dates related to Easter, as visible in calendar window."
 (if (and (> displayed-month 6) (not calendar-christian-all-holidays-flag))
     nil;; Ash Wednesday, Good Friday, and Easter are not visible.
   (let* ((century (1+ (/ displayed-year 100)))
          (shifted-epact        ;; Age of moon for April 5...
           (% (+ 14 (* 11 (% displayed-year 19));;     ...by Nicaean rule
                 (-           ;; ...corrected for the Gregorian century rule
                  (/ (* 3 century) 4))
                 (/    ;; ...corrected for Metonic cycle inaccuracy.
                  (+ 5 (* 8 century)) 25)
                 (* 30 century));;              Keeps value positive.
              30))
          (adjusted-epact       ;;  Adjust for 29.5 day month.
           (if (or (= shifted-epact 0)
                   (and (= shifted-epact 1) (< 10 (% displayed-year 19))))
               (1+ shifted-epact)
             shifted-epact))
          (paschal-moon       ;; Day after the full moon on or after March 21.
           (- (calendar-absolute-from-gregorian (list 4 19 displayed-year))
              adjusted-epact))
          (abs-easter (calendar-dayname-on-or-before 0 (+ paschal-moon 7)))
          (mandatory (list
            (list (calendar-gregorian-from-absolute (- abs-easter 46))
                  "Aschermittwoch")
            (list (calendar-gregorian-from-absolute (- abs-easter 2))
                  "Karfreitag")
            (list (calendar-gregorian-from-absolute abs-easter)
                  "Ostersonntag")
            (list (calendar-gregorian-from-absolute (+ abs-easter 1))
                  "Ostermontag")
            (list (calendar-gregorian-from-absolute (+ abs-easter 39))
                  "Christi Himmelfahrt")
            (list (calendar-gregorian-from-absolute (+ abs-easter 49))
                  "Pfingstsonntag")
            (list (calendar-gregorian-from-absolute (+ abs-easter 50))
                  "Pfingstmontag")
            (list (calendar-gregorian-from-absolute (+ abs-easter 60))
                  "Fronleichnam")
          ))
          (optional (list
            ;; Nothing for now
          ))
          (output-list
           (filter-visible-calendar-holidays mandatory)))
     (if all-christian-calendar-holidays
         (setq output-list
               (append
                (filter-visible-calendar-holidays optional)
                output-list)))
     output-list)))
;;
(setq holiday-general-holidays '(
  (holiday-fixed 01 01 "Neujahr")
  (holiday-fixed 05 01 "Tag der Arbeit")
  (holiday-fixed 10 03 "Tag der Deutschen Einheit")
  (holiday-fixed 12 31 "Sylvester")
))
(setq holiday-local-holidays '(
  (holiday-fixed 01 06 "Heilige Drei Knige")
  (holiday-float 11 03 03 "Bu- und Bettag")
  (holiday-fixed 12 06 "Nikolaustag")
))
(setq holiday-christian-holidays '(
  (holiday-fixed 11 01 "Allerheiligen")
  (holiday-fixed 12 24 "Heiligabend")
  (holiday-fixed 12 25 "1.Weihnachtstag")
  (holiday-fixed 12 26 "2.Weihnachtstag")
; (holiday-german-easter-etc)
))
(setq holiday-hebrew-holidays nil)
(setq holiday-islamic-holidays nil)
(setq holiday-solar-holidays nil)
(setq calendar-christian-all-holidays-flag t)

;;----------------------------------------------------------------------------
(provide 'gold_calendar)
