;;; This file is an addition to the GNU Emacs. GNU Emacs and the herein GOLD
;;; additions are distributed in the hope that it will be useful, but without
;;; ANY WARRANTY.  No author or distributor accepts responsibility to anyone
;;; for the consequences of using it or for whether it serves any particular
;;; purpose or works at all, unless he says so in writing.
;;; Everyone is granted permission to copy, modify and redistribute GNU Emacs,
;;; but only under the conditions described in the GNU Emacs General Public Li-
;;; cense.   A copy of this license is supposed to have been given to you along
;;; with GNU Emacs so you can know your rights and responsibilities.  It should
;;; be in a file named COPYING.  Among other things, the copyright notice and
;;; this notice must be preserved on all copies.
;;; Refer to the GNU Emacs General Public License for full details.
;;;
;;;############################################################################
;;;###  Improved completion of filenames etc. in minibuffer !               ###
;;;###  This file is included (direct- or auto-loaded) by the main entry    ###
;;;###  file >>> gold.el <<< => Refer to that file for more informations.   ###
;;;############################################################################
;;; Bug reports and suggestions are always welcome !
;;
(gold-debug "gold_complete")
;;
;; ??? minibuffer-setup-hook ???
;(defun define-cursor-history (local-map) " "
;      (define-key local-map [up]   'previous-history-element)
;      (define-key local-map [down] 'next-history-element)
;      (define-key local-map "\eOA" 'previous-history-element)
;      (define-key local-map "\eOB" 'next-history-element)
;)
;(if xemacs nil
;   (define-cursor-history minibuffer-local-map)
;   (define-cursor-history minibuffer-local-ns-map)
;   (define-cursor-history minibuffer-local-completion-map)
;   (define-cursor-history minibuffer-local-must-match-map)
;)
;;
;; Bind new functions to keys in minibuffer-local-maps:
;;-----------------------------------------------------
(add-hook 'term-setup-hook          ; Call key-definitions AFTER Terminal-Setup
          'minibuffer-local-keys t) ; because of the terminal-dependent
                                    ; substitute-key-definition commands !

(defun minibuffer-local-keys () "Rebind keys for minibuffer-local-maps"
      (auto-completion-keys minibuffer-local-completion-map)
      (auto-completion-keys minibuffer-local-must-match-map)
)
(defun auto-completion-keys (local-map)
   "Rebind automatic (electric) completion-keys for one LOCAL-MAP keymap."
   (substitute-key-definition 'self-insert-command 'self-insert-complete&list
                               local-map global-map)
   (substitute-key-definition 'delete-previous-char 'delete-previous-char&list
                               local-map global-map)
   (substitute-key-definition 'delete-backward-char 'delete-previous-char&list
                               local-map global-map)
   (substitute-key-definition 'new-line 'minibuffer-complete-and-exit
                               local-map global-map)
   (define-key local-map " "  'minibuffer-complete-auto-list)
   (define-key local-map "?"  'minibuffer-completion-help)
   (define-key local-map "\t" 'minibuffer-auto-complete&list)
)
;; Function definitions:
;;-----------------------
(defun temp-buffer-echo (string &optional seconds milliseconds)
"Temporarily display STRING in current-buffer (with NO keyboard-quit).
Optional SECONDS and MILLISECONDS say how long to wait (default is 1 second)."
  (let ((inhibit-quit t) (save (point)) end)
       (insert string) (setq end (point))
       (goto-char save)
       (if seconds (sit-for seconds milliseconds) (sit-for 1))
       (delete-region save end)
       (if quit-flag (if (string-match "^*Minibuf" (buffer-name))
                         (abort-recursive-edit) (keyboard-quit)))))
;;
(defvar minibuffer-auto-completion nil "Internal")
;;
(defun minibuffer-complete-auto-list ()
"Complete a minibuffer-completion and give an additional completion-list
(if not sole unique by first tried completion).
Also switches self-insert-commands and DEL to a special completion-mode."
  (interactive)
  (minibuffer-auto-complete&list)
  (setq minibuffer-auto-completion nil))
;
(defun minibuffer-auto-complete&list ()
"Complete a minibuffer-completion and give an additional completion-list
(if not sole unique by first tried completion).
Also switches self-insert-commands and DEL to a special completion-mode."
  (interactive)
  (setq minibuffer-auto-completion t)
  (let*((completion-auto-help nil)
        (original (buffer-string))
        (completion (try-completion original minibuffer-completion-table
                                             minibuffer-completion-predicate)))
       (if (and completion (stringp completion))
           (progn (if (equal original completion) nil
                      (erase-buffer) (insert completion))
                  (minibuffer-completion-list))
           (goto-char (point-max)))))
;;
(defun minibuffer-completion-list () "xxx"
  (interactive)
  (let ((completion (all-completions (buffer-string)
                                     minibuffer-completion-table
                                     minibuffer-completion-predicate)))
       (cond ((not completion)
              (temp-buffer-echo " [No match]"))
             ((and (= 1 (length completion)) minibuffer-auto-completion)
              (temp-buffer-echo " [Unique completion]"))
             (t
              (with-output-to-temp-buffer "*Completions*" ;; (princ "Test:")
                   (display-completion-list (sort completion 'string-lessp)))
              (setq this-command 'minibuffer-completion)))
       (goto-char (point-max))
))
;;
(defun self-insert-complete&list (arg)
"Just self-insert-command with additional completion-list."
      (interactive "p")
      (self-insert-command arg)
      (if (equal last-command 'minibuffer-completion)
          (if minibuffer-auto-completion (minibuffer-auto-complete&list)
                                         (minibuffer-completion-list))))
(defun delete-previous-char&list (num)
"Just delete-previous-char with additional completion-list."
      (interactive "p")
      (delete-previous-char num)
      (if (equal last-command 'minibuffer-completion)
          (let ((minibuffer-auto-completion nil))
               (minibuffer-completion-list))))
;;
;;----------------------------------------------------------------------------
(provide 'gold_complete)
