;;; This file is an addition to the GNU Emacs. GNU Emacs and the herein GOLD
;;; additions are distributed in the hope that it will be useful, but without
;;; ANY WARRANTY.  No author or distributor accepts responsibility to anyone
;;; for the consequences of using it or for whether it serves any particular
;;; purpose or works at all, unless he says so in writing.
;;; Everyone is granted permission to copy, modify and redistribute GNU Emacs,
;;; but only under the conditions described in the GNU Emacs General Public Li-
;;; cense.   A copy of this license is supposed to have been given to you along
;;; with GNU Emacs so you can know your rights and responsibilities.  It should
;;; be in a file named COPYING.  Among other things, the copyright notice and
;;; this notice must be preserved on all copies.
;;; Refer to the GNU Emacs General Public License for full details.
;;;
;;;############################################################################
;;;###  Help facilities (context-help and others) for gold.el !             ###
;;;###  This file is included (direct- or auto-loaded) by the main entry    ###
;;;###  file >>> gold.el <<< => Refer to that file for more informations.   ###
;;;############################################################################
;;; Bug reports and suggestions are always welcome !
;;
(gold-debug "gold_help")
;;-----------------------
;;  -> shell-mode: Test on error/warning line and switch to that buffer&line
;;                 Help for Manual-Entry if command-line ?
;;;###autoload
(defun context-help () ; Use help-level for double/triple-clicks ?
"Context sensitive Short-Cut to the normal help functions/commands."
  (interactive) (let ((pos (point)))
  (cond ((and (eq last-context-help-position pos)
              (eq last-command this-command)) ; toggling this behaviour !
                         (setq this-command 'dummy) (describe-mode))
        ;;
        (context-help-function (funcall context-help-function))
        ((equal major-mode 'client-mode)      (client-help-field))
        ((equal major-mode 'permas-aba-mode)  (help-aba-keyword))
        ((equal major-mode 'permas-dat-mode)  (help-dat-command))
        ((equal major-mode 'permas-log-mode)  (permas-logtime))
        ((equal major-mode 'doc-mode)         (show-page))
        ((equal major-mode 'jde-mode)         (jde-help-symbol))
        ((equal major-mode 'permas-nas-mode)  (help-nas-keyword))
        ((equal major-mode 'occur-mode)       (occur-mode-goto-occurrence))
        ((equal major-mode 'shell-mode)       (shell-context-help))
        ((equal major-mode 'timecard-mode)    (timecard-context-help))
        ((equal major-mode 'permas-pqa-mode)  (permas-pqa-get-help-file))
        ((equal major-mode 'permas-rsc-mode)  (help-rsc-variable))
        ((equal major-mode 'permas-stop-mode) (help-uci-command))
        ((equal major-mode 'permas-uci-mode)  (help-uci-command))
        ((equal major-mode 'permas-unv-mode)  (permas-unv-help-dataset))
        ((or (equal major-mode 'TeX-mode)
             (equal major-mode 'tex-mode)
             (equal major-mode 'plain-tex-mode)
             (equal major-mode 'texinfo-mode)
             (equal major-mode 'LaTeX-mode)
            (equal major-mode 'latex-mode)) (help-tex-command))
        ((equal (buffer-name) "*Help*")  (help-tex-command))
        ((equal (buffer-name) "*Manual Entry*")
           (let ((unread-command-events '(13))) (call-interactively 'man)))
        (t (describe-mode)))
  (setq last-context-help-position pos)))
(setq last-context-help-position nil); used for mouse-resetting of point
;;;
(defvar context-help-function nil
"Function called (if non-nil) to perform `context-help'.
Each buffer has its own value of this variable.")
;;
(make-variable-buffer-local 'context-help-function)
;;
(set-default 'context-help-function nil) ;; ?'describe-mode)
;;
;;-----------------------------------------------
;;;###autoload
(defun GOLD-help () "Extended Emacs-Help for GOLD package"
       (interactive)
       (call-interactively 'GOLD-HELP))
(defun GOLD-HELP (key) "Extended Emacs-Help for GOLD package"
      (interactive
"kHELP or A B C D E F G K M N R S T U V W $ & * = (or any key), ? for more help:")
      (cond
       ((equal key "?")
        (call-interactively 'GOLD-help-for-help) (message " "))
       ((GOLD-help-key)
        (call-interactively 'gold-keypad-help) (message " "))
       ((or (equal key "a") (equal key "A")) (help-aba-keyword t))
       ((or (equal key "b") (equal key "B")) (help-bulk-keyword t))
       ((or (equal key "c") (equal key "C")) (help-fortran-call t))
       ((or (equal key "d") (equal key "D")) (help-data-object t))
       ((or (equal key "e") (equal key "E"))
                            (call-interactively 'permas-error))
       ((or (equal key "f") (equal key "F"))
        (call-interactively 'describe-function))
       ((or (equal key "g") (equal key "G"))
                            (call-interactively 'show-permas-fmt-group))
       ((or (equal key "k") (equal key "K"))
        (call-interactively 'describe-key))
       ((or (equal key "m") (equal key "M")) (call-interactively 'man))
       ((or (equal key "n") (equal key "N"))
        (call-interactively 'GOLD-news-help))
       ((or (equal key "r") (equal key "R"))
        (call-interactively 'GOLD-RE-help))
       ((or (equal key "s") (equal key "S"))
        (call-interactively 'ispell-word))
       ((or (equal key "t") (equal key "T")) (help-tex-command t))
       ((or (equal key "u") (equal key "U")) (help-uci-command t))
       ((or (equal key "v") (equal key "V"))
        (call-interactively 'describe-variable))
       ((or (equal key "w") (equal key "W"))
        (call-interactively 'where-is)
        (sleep-for 2)) ;; Avoid message-delete through re-drawing screen !
       ((equal key "$")   (help-dat-command t))
       ((equal key "&")   (call-interactively 'describe-bindings))
       ((equal key "*")   (call-interactively 'describe-mode))
       ((equal key "=")   (call-interactively 'describe-char))
       ((equal key "\t")  (message " ")(edit-tab-stops))
       (t
        (let ((defn (key-binding key)))     ; Variable defn = Key description
        (if (or (null defn) (integerp defn))
            (message "%s is undefined" (key-description key))
            (message "%s runs the command %s"
                     (key-description key)
                     (if (symbolp defn) defn (prin1-to-string defn))))
        (sleep-for 2))) ;; Avoid message-delete through re-drawing screen !
))
(defun GOLD-help-key ()
     "Tests on pressed HELP key (default is PF2 and vt200-F15)"
      (or (equal key "\eOQ")  (equal key "\e[28~")
          (equal key [kp-f2]) (equal key [kp-divide])))

(defun GOLD-help-for-help ()
  "Typing the HELP key (PF2 key) you get the GOLD-Emacs help facility with:\n
? - Show this help text for GOLD-Emacs Help facilities.
A - Apropos command (list all commands matching a RE-search-string).
B - Show help for NASTRAN Bulk-Data keyword.
C - Show call-string for a PERMAS routine documented with INTeX .loc files
D - Show help for a PERMAS data structure (DMS object).
E - Show PERMAS Errors from GNATS-DataBase
F - Describe a Function (full documentation)
G - Show PERMAS Group Formats (FMT-Group)
K - Describe a key respectively key sequence (full documentation).
M - Display an Unix manual entry within a special buffer (man->buffer).
N - List News on GOLD-Emacs.
R - Show a help for Regular Expressions.
S - Spell-checking word at cursor position.
T - Show help for a INTeX or (La)TeX command respectively keyword.
U - Show help for a PERMAS UCI command.
V - Describe a Variable (full documentation).
W - Where is function (which keystroke invokes given function) ?
$ - Show help for a PERMAS DAT keyword (data input \$keyword).
* - Describe current major-mode (with special keybindings etc.)
= - Describe current character (coding, font, etc.)


Type the HELP key again to get help on GOLD-Keypad and more GOLD/HELP-Keys.
Type any other key to get a brief documentation of the function it invokes."
  (interactive)
  (save-window-excursion
       (switch-to-buffer "*Help*")
       (setq buffer-read-only nil) (erase-buffer)
       (insert (documentation 'GOLD-help-for-help))
       (call-interactively 'GOLD-HELP))
 )

(defun print-help-return-message (&optional function)
  "Display or return message saying how to restore windows after help command.
Computes a message and applies the argument FUNCTION to it.
If FUNCTION is nil, applies `message' to it, thus printing it."
  (and (not (get-buffer-window standard-output))
       (funcall (or function 'message)
                (substitute-command-keys
                 (if (one-window-p t)
                     (if pop-up-windows
                         "Type 'MAIN WINDOW' (keypad <'3'>) to remove help window."
                         "Type 'SWITCH BUFFER' (keypad <'4'>) followed by 'RET' to remove help window.")
                         "Type \\[switch-to-buffer-other-window] RET to restore old contents of help window.")))))

(defun GOLD-RE-help ()
"Regular Expressions (RE) for RE-search-strings, RE-replacements ...

. (dot) - Any single character except newline:  A.C matches AbC or A1C not AC
*       - Zero or more repeats of preceding char/RE:  .* matches any string
+       - One or more repeats of preceding char/RE:  A+ matches A or AAA not B
?       - Zero or one repeat of preceding char/RE: AB?C matches ABC AC not ABBC
[...]   - Any character in set: A[b1]+C matches AbC A1C or A1bb1C not AxC or AC
[^...]  - Any character not in set:  A[^a-z]C matches ABC or A1C not AbC or AxC
          Use '-' to define ranges like [1-9$%.] for $ % . or any digit 1...9
^    $  - Beginning/End of line:  ^[*cC] matches any non-blank FORTRAN comments
\\`   \\' - Beginning/End of buffer:  \\`.*[\\%] tests the first line for TeX
\\b      - Beginning or end of word (word break):  \\bAB matches ABcdef not xABc
\\B      - Not beginning or end of word:  \\BAB matches xABc or aAB not ABcdef
\\|      - Alternative OR:  A\\(bc\\|12\\)D matches AbcD or A12D (nothing else)
\\(...\\) - Grouping construct (see above):  A\\(bc\\)*D matches AbcbcD not AcbcbD
\\N      - Match the N-th grouping construct (N is any digit from 1...9)
          A\\(.*\\)B\1C matches ABC AxBxC or A-123-B-123-C but not AxByC

\\. \\* \\+ \\? \\$ \\^ \\\\ ... for special characters . * + ? $ ^ \\ [ ] ...
Use CTRL-Q followed by any keystroke or a octal number to insert any valid
ASCII-Character (e.g. inserting control codes, ESC-functions etc.)."
      (interactive)
      (save-window-excursion
           (switch-to-buffer "*Help*")
           (setq buffer-read-only nil) (erase-buffer)
           (insert (documentation 'GOLD-RE-help))
           (call-interactively 'GOLD-quit-help)))

(defun GOLD-quit-help (key)
      "Wait for any key to quit."
      (interactive "kType any key to quit:")
      (message " "))

;; keystring evaluation
;;----------------------
(defun keystring-context (keystring) "Get main context from keysting (if any).
with KEYSTRING in form \"CONTEXT;KEY SUBKEY1 SUBKEY2\" with optional substrings."
   (if (string-match "^\\([^;]+\\) *;" keystring)
       (match-string 1 keystring)))
;
(defun keystring-key (keystring) "Get main key from keysting (if any).
with KEYSTRING in form \"CONTEXT;KEY SUBKEY1 SUBKEY2\" with optional substrings."
   (let ((start (string-match "[^;]+$" keystring)))
        (if (string-match " *\\([^ ]+\\)" keystring start)
            (match-string 1 keystring))))
;
(defun keystring-subkey1 (keystring) "Get 1st subkey from keysting (if any).
with KEYSTRING in form \"CONTEXT;KEY SUBKEY1 SUBKEY2\" with optional substrings."
   (let ((start (string-match "[^;]+$" keystring)))
        (if (string-match " *[^ ]+ +\\([^ ]+\\)" keystring start)
            (match-string 1 keystring))))
;
(defun keystring-subkey2 (keystring) "Get 2nd subkey from keysting (if any).
with KEYSTRING in form \"CONTEXT;KEY SUBKEY1 SUBKEY2\" with optional substrings."
   (let ((start (string-match "[^;]+$" keystring)))
        (if (string-match " *[^ ]+ +[^ ]+ +\\([^ ]+\\)" keystring start)
            (match-string 1 keystring))))

;;****************************************************************************
;;*****  Help on PERMAS data input out of *um450* documentation !    *********
;;****************************************************************************
;;;###autoload
(defun help-dat-command (&optional query)
"Show help for PERMAS data input command of actual line (info from um450).
Ask interactively for keyword if QUERY is non-nil (short cut otherwise)."
 (interactive)
 (call-help-with-default 'help-dat-key 'dat-keystring-at-point
                          query "PERMAS input command"))
;;
(defun um450-document () "Document abbreviation used for UM450 helps."
  (if (stringp permas-doc-version)
      (concat "um450_" permas-doc-version) "um450"))
;;
(defvar dat-bracket-regexp
"$\\(end\\|enter\\|situation\\|structure\\|system\\|constraints\\|loading\\|modification\\|results\\|material\\|laminate\\|sandwich\\|fluid\\|flfsolid\\|gasket\\)\\b")
;;;
(defun dat-keystring-at-point () "Returns the keystring of actual DAT statement
in form \"BRACKET;KEY SUBKEY1 SUBKEY2\"."
  (condition-case () (save-excursion
  (let ((pos1)(pos2)(case-fold-search t)) ; Back to most-last $keyword
       (end-of-line) (re-search-backward "^ *[$][a-z]" nil t)
       (skip-chars-forward " ") (setq pos1 (point))
       (if (looking-at "[$][a-z]")
           (progn (skip-chars-forward "$0-9A-Za-z") (setq pos2 (point))
                  (if (< pos1 pos2)
                      (let ((bracket (dat-bracket-at-point))
                            (key     (upcase (buffer-substring-no-properties pos1 pos2)))
                            (subkey  (subkeys-at-point)))
                           (if (string-match dat-bracket-regexp key) (eval key)
                               ;; not a bracket command
                               (concat (if bracket (concat bracket ";"))
                                   key (if subkey (concat " " subkey)))))
                      (eval "[????]")))
           "[???]")))))
;
(defun dat-bracket-at-point () "Returns the DAT Bracket at point."
  (let ((bracket)(save (point))(case-fold-search t))
       (skip-chars-forward "0-9A-Za-z")
       (if (re-search-backward (concat "^ *" dat-bracket-regexp) nil t)
           (progn (setq bracket (upcase (match-string 1)))
                  (if (string-equal "END" bracket) (setq bracket nil))))
       (goto-char save) (eval bracket)))

;;****************************************************************************
;;*****  Help on UCI commands out of *um450* documentation !         *********
;;****************************************************************************
;;;###autoload
(defun help-uci-command (&optional query)
"Show help for UCI command of actual line (info from um450 documentation).
Ask interactively for keyword if QUERY is non-nil (short cut otherwise)."
 (interactive)
 (call-help-with-default 'help-uci-key 'uci-keystring-at-point
                          query "UCI command"))
;;
(defvar uci-section-regexp
"\\(return\\|exec\\|export\\|input\\|license\\|print\\|select\\|startup\\|user\\)")
;;;
(defun uci-keystring-at-point () "Returns the keystring of actual UCI statement
in form \"SECTION;KEY SUBKEY1 SUBKEY2\"."
  (condition-case () (save-excursion
  (let ((pos1)(pos2)(case-fold-search t))
       (re-search-backward "^\\|;" nil t) ; Back to SOL or most-last ;-char
       (skip-chars-forward " ;") (setq pos1 (point))
       (if (looking-at "[$]") (dat-keystring-at-point)
       (if (looking-at "[a-z]") ; "[$a-z]"
           (progn (skip-chars-forward "0-9A-Za-z") (setq pos2 (point))
                  (if (< pos1 pos2)
                      (let ((section (uci-section-at-point))
                            (key     (upcase (buffer-substring-no-properties pos1 pos2)))
                            (subkey  (subkeys-at-point)))
                           (if (string-match uci-section-regexp key) (eval key)
                               ;; not a section entry
                               (concat (if section (concat section ";"))
                                   key (if subkey (concat " " subkey)))))
                      (eval "[????]")))
           (if (re-search-backward "[$a-z].*$" nil t) ; look for previous line
               (uci-keystring-at-point) "[???]" )))))))
;
(defun uci-section-at-point () "Returns the UCI Section at point."
  (let ((section)(save (point))(case-fold-search t))
       (skip-chars-forward "0-9A-Za-z")
       (if (re-search-backward (concat "^[^!]*\\b" uci-section-regexp "\\b") nil t)
           (progn (setq section (upcase (match-string 1)))
                  (if (string-equal "RETURN" section) (setq section nil))))
       (goto-char save) (eval section)))
;
(defun subkeys-at-point () "Returns the next 2 Subkeys/parameter after point."
   (if (looking-at " *\\([a-zA-Z0-9]+ *[a-zA-Z0-9]*\\)")
       (upcase (match-string 1))))
;  (let ((subkey)(save (point))(case-fold-search t))
;       (if (looking-at " *\\([a-zA-Z0-9]+\\)")
;           (progn (setq subkey (upcase (match-string 1)))
;                  (goto-char (match-end 1))
;                  (if (looking-at " *=") (setq subkey nil)))) ;; ignore param = ?
;       (goto-char save) (eval subkey)))
;;
(defun help-dat-key (keystring) "Display help for DAT KEYword." (help-um450-key "DAT" keystring))
(defun help-uci-key (keystring) "Display help for UCI KEYword." (help-um450-key "UCI" keystring))
;;
(defun help-um450-key (keytype keystring) "Display help for um450 UCI/DAT KEYword."
  (let ((context (keystring-context keystring))
        (key     (keystring-key keystring)) (mode mode-name)
        (subkey1 (keystring-subkey1 keystring))
        (subkey2 (keystring-subkey2 keystring))) ;; param1 param2 ????
  (cond ((equal major-mode 'permas-stop-mode) (setq mode "StopFile"))
        ((equal major-mode 'permas-step-mode) (setq mode "StepFile"))
  )
  (let ((original-buffer (current-buffer)) (name "*um450-chi*")
        (KEY (if (string-match "^[$]" key) (substring key 1) key)) ; cut $key
        (TYP (if (string-match "^[$]" key) "DAT" keytype)))
       (um450-chi) (set-buffer name) (goto-char (point-min))
       (if (or (re-search-forward (format "^%s %s %s.*@Ctx>%s;%s" KEY subkey1 subkey2 TYP context) nil t)
               (re-search-forward (format "^%s %s %s.*@Ctx>%s"    KEY subkey1 subkey2 TYP) nil t)
               (re-search-forward (format "^%s %s.*@Ctx>%s;%s"    KEY subkey1 TYP context) nil t)
               (re-search-forward (format "^%s %s.*@Ctx>%s"       KEY subkey1 TYP) nil t)
               (re-search-forward (format "^%s.*@Ctx>%s;%s"       KEY TYP context) nil t)
               (re-search-forward (format " %s %s.*@Ctx>%s;%s"    KEY subkey1 TYP context) nil t)
               (re-search-forward (format " %s %s.*@Ctx>%s"       KEY subkey1 TYP) nil t)
               (re-search-forward (format "^%s.*@Ctx>%s"          KEY TYP) nil t)
               (re-search-forward (format " %s.*@Ctx>%s"          KEY TYP) nil t)
               (re-search-forward (format "^%s .*@Ctx>"           KEY) nil t)
               (re-search-forward (format " %s .*@Ctx>"           KEY) nil t)
               (re-search-forward (format "%s.*@Ctx>"             KEY) nil t)
               (re-search-forward (format "%s.*@Ctx>"         context) nil t)
               (re-search-forward (format "%s.*@Ctx>"            mode) nil t))
           (chi-view-page)
           (beep) (message (format
                  "No such %s-Command \"%s\" (search failed)" TYP KEY)))
       (set-buffer original-buffer))))
;;
(defun um450-chi () "Load context-help-index for um450 to *um450-chi* buffer."
  (let ((name "*um450-chi*"))
       (if (get-buffer name) (set-buffer name)
           (unconditional-new-buffer name nil)
           (insert-file (permas-doc-file "um450" ".chi"))
           (set-buffer-modified-p nil) (setq buffer-read-only t))))
;;
(defun chi-view-um450-tty ()
  (if (load-document (um450-document))
      (let ((pmin (point-min)) (pmax (point-max))
            (KEY (concat "^ *|+  +" (if (not uci) "[$]") (upcase key) ".*|")))
           (goto-char pmin)
           (re-search-forward (concat "^" locpage "\\|" locpage "$") nil t)
           (if (or (re-search-backward KEY (max pmin (- (point)  10000)) t)
                   (re-search-forward  KEY (min pmax (+ (point)   5000)) t)
                   (re-search-backward KEY (max pmin (- (point) 100000)) t)
                   (re-search-backward KEY (min pmax (+ (point)  50000)) t))
                (with-output-to-temp-buffer "*Help*"
                     (show-uci-syntax (upcase key))
                     (highlight-buffer nil "*Help*"
                                           'permas-help-mode 'auto))
                (beep) (message (concat "No such " (if uci "UCI" "Data Input")
                                     " command \"" key "\" (search failed)")))
            (set-buffer original-buffer))))
;;
(defun show-uci-syntax (keystring) ""
  (let ((key (keystring-key keystring)))
  (let ((pos (point)) (indent (string-match "[^ |]" (buffer-linestring))))
       (if (not uci) (re-search-backward "^ *$" nil t))
       (print-headline-and-page (um450-document)
          (if uci "UCI keyword" "DAT input") key t)
       (princ "\n") (goto-char pos) (print-syntax-block indent t))))
;
(defun print-syntax-block (indent &optional frame)
  "Read syntax block around point and print it in compressed form.
   Print with frame if optional FRAME is non-nil."
  (if frame (princ (concat "+" (make-string 77 ?-) "+\n")))
  (let (line1 line2) (search-backward "____________________" nil t)
       (forward-line 1) (setq line1 (buffer-linestring))
       (while (not (string-match "____________________" line1))
              (forward-line 1) (setq line2 (buffer-linestring))
              (if (string-match "[^ |] *|" line1) (progn
                  (if (test-subscript-line line1 line2) (progn
                      (setq line1 (include-subscript-line line1 line2))
                      (forward-line 1) (setq line2 (buffer-linestring))))
                  (setq line1 (string-to-length (substring line1 indent) 74))
                  (if frame (princ (concat "|   " line1 "|\n"))
                            (princ (concat "    " line1 "\n")))))
              (setq line1 line2)))
  (if frame (princ (concat "+" (make-string 77 ?-) "+"))))

(defun test-subscript-line (line1 line2)
"Test wether line2 can be taken as TeX-Subscript of line1."
  (let ((c1)(c2)(i (min (length line1) (length line2)))(ret t))
       (while (and (< 0 i) ret)  (setq i (- i 1))
              (setq c1 (substring line1 i (+ i 1)))
              (setq c2 (substring line2 i (+ i 1)))
              (if (not (or (equal c1 c2) (equal c1 " ") (equal c2 " ")))
                  (setq ret nil)))
       (eval ret)))
;
(defun include-subscript-line (line1 line2)
"Update blank-chars of line1 with TeX-Subscripts at corresponding line2."
  (let ((c)(i (min (length line1) (length line2))))
       (while (< 0 i)  (setq i (- i 1))
              (setq c (substring line2 i (+ i 1)))
              (if (equal (substring line1 i (+ i 1)) " ")
              (if (not (or (equal c "|") (equal c "-") (equal c "_")))
                  (setq line1 (change-char-in-string line1 i c)))))
       (eval line1)))

;;************************************************************
;;*****  Help on PERMAS data objects (alpha test phase)  *****
;;************************************************************
(defun help-data-object (&optional query) "Dummy" (interactive)
  (beep) (message "This command moved to Gold-D !"))

;;*******************************************************
;;*****  Help on PERMAS calls through *Call List*   *****
;;*******************************************************
(defun call-at-point ()
"Returns a string which serves as default for search-list-of-calls."
  (save-excursion
   (let ((save (point)) (min-pos (min (point-min) (- (point) 100))))
        (save-excursion (if (re-search-backward
             "^[ 0-9][ 0-9][ 0-9][ 0-9][ 0-9] \\|call\\|routine" min-pos t)
                            (setq min-pos (match-end 0))))
        (re-search-backward "(" min-pos t)
        (word-at-point))))
;;
;;;###autoload
(defun help-fortran-call (&optional query)
"Search first routine within *Call List* that matches funsub
(create the *Call List* out of INTeX documentation at first call).
Ask interactively for keyword if QUERY is non-nil (short cut otherwise)."
 (interactive) (call-help-with-default 'search-call-list 'call-at-point
                                        query "Name of Function/Subroutine"))
;;
(defun list-of-calls ()
      "Switch to buffer with Call-List (create at first call)"
      (if (get-buffer "*Call List*") (set-buffer "*Call List*")
          (message "Creating *Call List*")
          (unconditional-new-buffer "*Call List*" nil) (advance-direction)
          (if (file-readable-p (concat permas-doc-dir "calls_intern.chi"))
              (insert-file-contents (concat permas-doc-dir "calls_intern.chi"))
              (insert-file-contents (concat permas-doc-dir "calls.chi")))
          (message "Creating *Call List*   Done")
          (set-buffer-modified-p nil)
          (setq buffer-read-only t)))
;;
(defun search-call-list (funsub)
      "Search for FUNSUB in *Call List* and display help."
   (let ((actual-buffer (current-buffer)))
        (list-of-calls) (set-buffer "*Call List*") (goto-char (point-min))
        (if (re-search-forward (concat "^" funsub "[ </]") nil t)
            (progn (show-help-for-call)
                   (end-of-line)
                   (if (re-search-forward (concat "^" funsub "[ <]") nil t)
                       (progn (beep)
                 (message (concat ">>> WARNING: Multiple " (upcase funsub)
                          " definitions => See Buffer *Call List* ! <<<")))))
            (beep) (message (concat "No such Function/Subroutine \""
                                     funsub "\" (search failed)")))
        (set-buffer actual-buffer)))
;;
(defun show-help-for-call () ""
  (let*((FUNSUB (upcase funsub))
        (line (buffer-linestring))
        (ttyhelp (concat permas-doc-dir "TTYcalls_intern/" FUNSUB ".tty"))
        (call (or (chi-get-field line) "???"))
        (type "SUBROUTINE") ;; (t "MODULE (type unknown) "))
        (ctx   (or (chi-get-field line "Ctx") "PERMAS"))
        (doc   (or (chi-get-field line "Doc") "???:??(??)"))
        (title (or (chi-get-field line "Tit") "No title-string !"))
        (indent (+ 11 (length FUNSUB))))
;;
  (or (file-readable-p ttyhelp)
      (setq ttyhelp (concat permas-doc-dir "TTYcalls/" FUNSUB ".tty")))
  (if (not (string-match "[!:]$" title)) (setq title (concat title ":")))
  (if (string-match "@Typ>Module" line) (setq type "MODULE"))
  (if (string-match "@Typ>Common" line) (progn (setq type "COMMON")
      (setq indent (- indent 3))
      (setq call (concat "/" call))
      (setq FUNSUB (concat "/" FUNSUB "/"))))
  (if (string-match "^[^ ()/]*<-" call) (progn (setq type "FUNCTION")
      (setq call (concat "<-" (replace-regexp-in-string "<- *(" " (" call)))
      (setq FUNSUB (concat "<-" FUNSUB))))
  (with-output-to-temp-buffer "*Help*"
      (princ (format "%-63s%s\n%s\n%s\n" title ctx
                     (make-string (length title) ?-)
                     (chi-see-page doc)))
      (setq line (format "%s %s" type call))
      (if (string-match "\\\\n" line)
          (progn (setq line (concat (substring line 0 (match-beginning 0))
                              "\n " (make-string (length type) ? )
                                    (make-string (length FUNSUB) ?=)
                               "  " (substring line (match-end 0))))
                  (setq line (replace-regexp-in-string "\\\\n"
                              (concat "\n  " (make-string indent ? )) line))
                  (princ line))
         (if (and (< (window-width) (length line))
                  (string-match "," line (- (window-width) 10)))
             (princ (format "%s,\n %s%s  %s\n"
                            (substring line 0 (match-beginning 0))
                            (make-string (length type) ? )
                            (make-string (length FUNSUB) ?=)
                            (substring line (match-end 0))))
             (princ (format "%s\n%s %s" line
                            (make-string (length type) ? )
                            (make-string (length FUNSUB) ?=)))))
     (if (file-readable-p ttyhelp)
         (save-excursion (set-buffer "*Help*") (princ "\n\n")
              (let ((start (point)))
                   (insert-file ttyhelp) (goto-char start)
                   (if (re-search-forward "^ *[\n]" nil t 2)
                       (kill-region start (match-end 0)))
                   (goto-char start)
                   (if (re-search-forward "^[0-9]" nil t)
                       (kill-region (match-beginning 0) (point-max)))))
      (or (equal type "COMMON") (equal type "MODULE")
          (let ((newline (concat "\n" (make-string indent ? ) "  ")))
               (princ (concat newline "-> means INPUT argument"
                              newline "<> means TRANSIENT argument"
                              newline "<- means OUTPUT argument"
                              newline "<? either TRANSIENT or OUTPUT argument")
       ))))
      (highlight-buffer nil "*Help*" 'permas-help-for-call 'auto)
      (print-help-return-message))))
;;;
(defun fold-line (line indent) " "
  (if (and (< (window-width) (length line))
           (string-match "," line (- (window-width) 10)))
      (format "%s\n%s" (substring line 0 (match-beginning 0))
                       (fold-line (concat (make-string indent ? )
                                          (substring line (match-end 0)))
                                  indent))
      line))
;;;
;;;###autoload
(defun insert-list-of-calls () ""   (interactive)
   (let ((enable-recursive-minibuffers t) funsub)
        (setq funsub (read-no-blanks-with-default
              "Name of Function/Subroutine" "TUPRD"))
   (let ((actual-buffer (current-buffer)))
        (list-of-calls) (goto-char (point-min))
        (if (re-search-forward (concat "^" funsub "[ <]") nil t)
            (insert-call)
            (beep) (set-buffer actual-buffer) (insert ";CA")
            (message (concat "No such Function/Subroutine \""
                                     funsub "\" (search failed)")))
   )))
;;
(defun insert-call () ""
  (let*((FUNSUB (upcase funsub))
        (line (buffer-linestring))
        (call (or (chi-get-field line) "???"))
        (function (string-match "^[^(]+<- *(" call)))
      (if (string-match "^[^(]+(" call)
          (setq call (substring call (- (match-end 0) 1)))
          (setq call "()"))
      (set-buffer actual-buffer)
      (if function (insert (concat "= " FUNSUB call))
                   (insert (concat "CALL " FUNSUB call)))
      (search-backward "(")))

;;****************************************************************************
;;*****  `chi'-Help-Utilities                                           *****
;;****************************************************************************
;(defun chi-get-title (line) ""
;  (or (chi-get-field line "Tit") "No title found !"))
;;
(defun chi-get-field (line &optional key) ""
  (if (and key (stringp key) (< 0 (length key)))
      (if (string-match (format " @%s>" key) line)
          (let*((ia (+ 6 (match-beginning 0)))
                (ie (string-match " @[^ ]+>\\|$" line ia)))
               (if (< ia ie) (substring line ia ie) nil))
          nil)
      (let ((ie (string-match " @[^ ]+>\\|$" line)))
           (if (< 1 ie) (substring line 0 ie) nil))
))
(defun chi-get-doc (line) ""
  (let ((doc (or (chi-get-field line "Doc") "???:??(??)")))
       (if (string-match "^\\([^(:)]*\\):\\([^(:)]*\\)(?\\([^(:)]*\\)" doc)
           (progn (setq chi-doc     (match-string 1 doc))
                  (setq chi-page    (match-string 2 doc))
                  (setq chi-abspage (match-string 3 doc))
))))
(defun chi-see-page (doc-field) ""
  (let ((title) (abbrev "???") (page "??-??"))
       (if (string-match "^\\([^(:)]*\\):\\([^(:)]*\\)" doc-field)
           (progn (setq abbrev  (match-string 1 doc-field))
                  (setq page    (match-string 2 doc-field))))
       (setq title (or (cdr (assoc abbrev document-alist))
                       (cdr (assoc (downcase abbrev) document-alist))
                       "UNKNOWN DOCUMENT TITLE"))
       (format "See page %s of \"%s\"\n" page title)))
;; (if (string-match "^\\([^(:)]*\\):\\([^(:)]*\\)(?\\([^(:)]*\\)" doc-field)

(defun chi-doc-title (name)
   "Return the title of document NAME from current chi-buffer."
   (save-excursion (goto-char (point-min))
   (if (re-search-forward (concat "@Doc>" name ".*@Tit>\\(.*\\) @Rev") 10000 t)
       (buffer-substring-no-properties (match-beginning 1) (match-end 1)) name)))

(defun chi-title ()
  "Get @Tit> entry for current line of Context-Help-Index."
  (save-excursion
  (if (re-search-forward (concat "@Tit>\\([^@]*\\) *$") nil t)
      (buffer-substring-no-properties (match-beginning 1) (match-end 1)))))

(defun chi-view-page ()
  "View document page for current line of Context-Help-Index."
  (let ((chi-line (buffer-linestring)) (ia) (ie) (name)
        (locpage) (abspage) (pdffile) (title))
  ;
  ; Get PDF-file name
  (setq ia (+ 5 (string-match "@Doc>" chi-line)))
  (setq ie (string-match ":" chi-line ia))
  (setq name (substring chi-line ia ie))
  (setq pdffile (permas-doc-file name ".pdf"))
  ;
  ; Get document title
  (setq title (chi-doc-title name))
  ;
  ; Get local page name
  (setq ia (+ 1 ie)) (setq ie (string-match "(" chi-line ia))
  (setq locpage (substring chi-line ia ie))
  ;
  ; Get absolute page number
  (setq ia (+ 1 ie)) (setq ie (string-match ")" chi-line ia))
  (setq abspage (substring chi-line ia ie))
  ;
  ; Load pdffile and view page
  (if pdfview-program
      (progn (message "PDFview for page %s (%s) of %s" locpage abspage title)
             (cond ((string-match "xpdf"     pdfview-program) (xpdf     pdffile abspage title))
                   ((string-match "qpdfview" pdfview-program) (qpdfview pdffile abspage title))
                   ((string-match "evince"   pdfview-program) (evince   pdffile abspage title))
                   ((string-match "okular"   pdfview-program) (okular   pdffile abspage title))
                   ((string-match "acro"     pdfview-program) (acroread pdffile abspage title))))
      (message "TTYview for page %s (%s) of %s" locpage abspage title)
      (chi-view-um450-tty)
)))
;;

;;****************************************************************************
;;*****  Help for integer dumps etc. (integer conversion)   ******************
;;****************************************************************************
;;*****    65537     1145457748 1599293780  -538976288 for tests !
(defun integer-display ()
"Read a decimal integer and display its string-, i2pack-, hex-, octal- and binary
representation, e.g. interpreting dumps like ASCII coded integers."
   (interactive)
   (let ((enable-recursive-minibuffers t))
         (ihex-display-calc (read-no-blanks-with-default
                            "Integer value" (ihex-at-point)))))

(defun integer-display-key ()
"Take next available integer and display its string-, i2pack-, hex-, octal- and binary
representation, e.g. interpreting dumps like ASCII coded integers."
   (interactive) (ihex-display-calc (ihex-at-point)) (word-back-forward 1))

(defun integer-display-calc (dec)
   "Calculation and message for integer-display and integer-display-key."
   (let ((dec10 (concat "10#" dec))(hex)(oct)(bin)(txt))
        (calc-eval '("1" calc-hex-radix)) ; Trick-Loading
        (calc-hex-radix)    (setq hex (substring (calc-eval dec10) 3))
        (calc-octal-radix)  (setq oct (substring (calc-eval dec10) 2))
        (calc-binary-radix) (setq bin (substring (calc-eval dec10) 2))
        (setq txt (or (hexascii-to-string hex) "???"))
        (message "%s = %s \"%s\" \$%s %%%s %s"
                  dec (i2pack-to-string dec) txt hex oct bin)))

(defun ihex-display-calc (ihex)
   "Calculation and message for integer-display and integer-display-key."
   (let ((dec)(hex)(oct)(bin)(txt))
        (calc-eval '("1" calc-hex-radix)) ; Trick-Loading
        (calc-hex-radix)     (setq hex (substring (calc-eval ihex) 3))
        (calc-octal-radix)   (setq oct (substring (calc-eval ihex) 2))
        (calc-binary-radix)  (setq bin (substring (calc-eval ihex) 2))
        (calc-decimal-radix) (setq dec (calc-eval ihex))
        (setq txt (or (hexascii-to-string hex) "???"))
        (message "%s = %s \"%s\" \$%s %%%s %s"
                  dec (i2pack-to-string dec) txt hex oct bin)))
;
(defun hexascii-to-string (hex)
"Convert a hexadecimal ascii-string into its character-string representation."
   (let ((string "")(ix)(jx)(s1)) (calc-decimal-radix) (setq jx (length hex))
        (while (< 0 jx)
               (setq ix (max (- jx 2) 0))
               (setq s1 (string-to-int
                        (calc-eval (concat "16#" (substring hex ix jx)))))
               (if (or (< s1 32) (> s1 126)) (setq s1 63))
               (setq string (concat (char-to-string s1) string))
               (setq jx ix))
    (eval string)))
;
;;*****    65537     1145457748 1599293780  -538976288 for tests !
(defun i2pack-to-string (dec)
"Convert a I2PACKed integer into its character-string representation."
   (let ((string "")(i2max "65536")(ix)(jx)) (calc-decimal-radix)
        (if (string= (calc-eval (concat dec ">2^32")) "1")
                     (setq i2max (calc-eval "2^32")))
        (setq ix (calc-eval (concat "floor(" dec "/" i2max)))
        (setq jx (calc-eval (concat dec "-" i2max "*" ix)))
        (setq string (concat "(" ix "," jx ")"))
    (eval string)))
;
(defun integer-3ascii ()
"Take next 3 available ASCII-integers and display its character-representation.
E.g. useful for 3S4-Coded integer names from decimal dumps like NAME(1...3) !"
   (interactive)
   (let ((dec1 0)(dec2 0)(dec3 0))
        (setq dec1 (integer-at-point)) (word-back-forward 1)
        (setq dec2 (integer-at-point)) (word-back-forward 1)
        (setq dec3 (integer-at-point)) (word-back-forward 1)
        (message (concat "ASCII-Name3 of " dec1 "," dec2 "," dec3 " is \""
               (decascii-to-string dec1) (decascii-to-string dec2)
               (decascii-to-string dec3) "\""))))
;
(defun integer-ascii ()
"Take next available ASCII-integer and display its character-representation."
   (interactive)
   (let ((dec1 0))
        (setq dec1 (integer-at-point)) (word-back-forward 1)
        (message (concat "ASCII-Name of " dec1 " is \""
                 (decascii-to-string dec1) "\""))))

(autoload 'calc-hex-radix "calc-bin")
(defun decascii-to-string (dec)
   "Convert a decimal ascii-string into its character-string representation."
   (calc-eval '("1" calc-hex-radix)) (calc-hex-radix) ; Trick-Loading
   (hexascii-to-string (substring (calc-eval (concat "10#" dec)) 3)))
;;*****                     1145457748  1599293780  538976288  for tests !

;;****************************************************************************
;;*****  Help on INTeX- respectively (La)TeX-Commands/Keywords !    **********
;;****************************************************************************
(defun help-tex-command (&optional query)
"Show help for INTeX or (La)TeX command/keyword (info from tex-macros.tty).
Ask interactively for keyword if QUERY is non-nil (short cut otherwise)."
 (interactive) (call-help-with-default 'help-tex-key 'texkey-at-point
                                 query "INTeX or (La)TeX command/keyword"))
;
(defun texkey-at-point ()
"Returns a string which serves as default for help-tex-command."
  (condition-case () (save-excursion
      (re-search-backward "[\\]" nil t) ;;(forward-char 1)
      (if (looking-at "[\\]\\(B\\|beg\\|begin\\|E\\|end\\) *{")
          (search-forward "{" nil t))
      (buffer-substring-no-properties
                (point) (progn (forward-char 1)  ;; One-Char-Symbols !
                        (skip-chars-forward "a-zA-Z0-9@*_$") (point))))
    (error nil)))
;
(defun help-tex-key (key) "Display help for TeX KEYword."
   (let ((actual-buffer (current-buffer))
         (KEY (concat "^@... +[^ ]+ +" key " "))
         (config (current-window-configuration)))
        (or (and (fboundp 'latex-help)
                 (condition-case nil (latex-help key t)
                           (error (set-window-configuration config) nil)))
        (if (load-document "tex-macros") (progn (goto-char (point-min))
            (if (re-search-forward KEY nil t)
                (with-output-to-temp-buffer "*Help*"
                     (show-tex-syntax)
                     (while (re-search-forward KEY nil t)
                            (princ (concat (make-string 79 ?- ) "\n"))
                            (message (concat "Multiple entries found for \""
                                              key "\" keyword !"))
                            (show-tex-syntax)))
                (beep) (message (concat "No such TeX command/keyword \""
                                         key "\" (search failed)")))))
            (set-buffer actual-buffer))))
;
(defun show-tex-syntax () "Read and print actual TeX-Help."
  (let*((line (buffer-linestring))
        (type1 (substring line 1 2))
        (type2 (substring line 2 3))
        (type3 (substring line 4 5))
        (ie (string-match " " line 5))
        (environment (substring line 5 ie))
        (ia (string-match "[^ ]" line ie))
        (ie (string-match " " line ia))
        (name (substring line ia ie))
        (ia (string-match "[^ ]" line ie))
        (ie (string-match " *!\\| *$" line ia))
        (arguments nil))
       (if (and ia ie (< ia ie)) (setq arguments (substring line ia ie)))
       (princ "Keyword \"")
       (if (string-match type2 "Mm") (princ "\\"))
       (princ (concat name "\" is used as "))
       (cond ((string-match type1 "Dd") (princ "German-INTeX-"))
             ((string-match type1 "Gg") (princ "German-LaTeX-"))
             ((string-match type1 "Ii") (princ "INTeX-"))
             ((string-match type1 "Ll") (princ "LaTeX-"))
             ((string-match type1 "Pp") (princ "plain TeX-"))
             ((string-match type1 "Tt") (princ "build-in TeX-"))
             ((string-match type1 "Xx") (princ "illegal TeX-"))
             (t (princ "???-")))
       (cond ((string-match type1 "Aa") (princ "Accent-Macro"))
             ((string-match type2 "Cc") (princ "Counter"))
             ((string-match type2 "Dd") (princ "Dimension"))
             ((string-match type2 "Ee") (princ "Environment"))
             ((string-match type2 "Ff") (princ "FontName"))
             ((string-match type2 "Ii") (princ "FontSize-Macro"))
             ((string-match type2 "Mm") (princ "Macro"))
             ((string-match type2 "Pp") (princ "PageStyle"))
             ((string-match type2 "Oo") (princ "OptionStyle"))
             ((string-match type2 "Ss") (princ "DocumentStyle"))
             ((string-match type2 "Tt") (princ "FontType-Macro"))
             ((string-match type2 "Zz") (princ "PageSize-Macro"))
             ((equal        type2 "*")  (princ "Symbol-Macro"))
             (t (princ "Keyword")))
       (if arguments (princ " with following arguments")
                     (princ " with NO arguments"))
       (if (string< "-" environment )
           (princ (concat "\n(applicable in \"" environment
                          "\"-Environment only)")))
       (princ ":\n\n=> ")
       (cond ((string-match type2 "Ee") (princ "\\B{"))
             ((string-match type2 "Pp") (princ "\\pagestyle{"))
             ((string-match type2 "Oo") (princ "\\documentstyle["))
             ((string-match type2 "Ss") (princ "\\documentstyle{"))
             ( t (princ "\\")))
       (princ name)
       (if (string-match type2 "Oo") (princ "]{mainstyle}"))
       (if (string-match type2 "EePpSs") (princ "}"))
       (if arguments (princ arguments))
       (if (string-match type2 "Ee") (princ (concat "\n   \\E{" name "}")))
       (princ "\n\n")
       ;; Now look for comment lines
       (setq ia (string-match " ! ." line 10))
       (if ia (princ (concat (substring line (+ ia 3)) "\n")))
       (forward-line 1) (setq line (buffer-linestring))
       (while (string-match "^ *! ." line)
              (setq ia (string-match "! " line))
              (princ (concat (substring line (+ ia 2)) "\n"))
              (forward-line 1) (setq line (buffer-linestring)))
))

;;****************************************************************************
;;****** Global used interface and utilities to help functions       *********
;;****************************************************************************
(defun call-help-with-default (help-function default-function query query-text)
"Call HELP-FUNCTION with default value obtained by DEFAULT-FUNCTION.
If third argument QUERY is non-nil, then ask interactively for the keyword
(with QUERY-TEXT as question). Do not ask if QUERY is nil (just take default)."
 (if query (let ((enable-recursive-minibuffers t) key)
                (if (stringp default-function)
                    (setq key (read-no-blanks-with-default query-text
                                                   default-function))
                    (setq key (read-with-default query-text
                                          (funcall default-function))))
                (funcall help-function key))
           (funcall help-function (funcall default-function))))

;;****************************************************************************
;;****** Context-Update function(s) & variable(s)                    *********
;;****************************************************************************
;;;###autoload
(defun context-update () "Context sensitive update of data around point"
  (interactive)
  (if context-update-function
     (funcall context-update-function)
     (beep)
     (message "Sorry: No `context-update-function' for current buffer.")))
;;
(defvar context-update-function 'update-day-month-year
"Function called (if non-nil) to perform `context-update'.
Each buffer has its own value of this variable.")
;;
(make-variable-buffer-local 'context-update-function)
;;
;;(set-default 'context-update-function 'update-day-month-year)

;;****************************************************************************
;;****** Context-Work function(s) & variable(s)                      *********
;;****************************************************************************
;;;###autoload
(defun context-work () "Context sensitive 'work' for data around point"
  (interactive)
  (if context-work-function
     (funcall context-work-function)
     (beep)
     (message "Sorry: No `context-work-function' for current buffer.")))
;;
(defvar context-work-function 'update-day-month-year
"Function called (if non-nil) to perform `context-work'.
Each buffer has its own value of this variable.")
;;
(make-variable-buffer-local 'context-work-function)
;;

;;;---------------------------------------------------------------------------
;;************************************************************
;;*****  Help on PERMAS data objects (alpha test phase)  *****
;;************************************************************
;;;###autoload
(defun help-data-object (&optional query)
"Search and display help for data object (create *Data List*).
Ask interactively for keyword if QUERY is non-nil (short cut otherwise)."
 (interactive) (call-help-with-default 'search-data-list 'word-at-point
                                        query "Name of Data Object"))
;;
(defun completing-data-name (string predicate type) " "
  (save-window-excursion
    (list-of-data) (set-buffer "*Data List*") (goto-char (point-min))
      ;(re-search-forward (concat "^" string) nil t)
    '(("MESH" 1) ("MESH.NBD" 2))
))
;;
(defun list-of-data () "Switch to buffer with Data-List (create at first call)"
  (if (get-buffer "*Data List*") (set-buffer "*Data List*")
      (unconditional-new-buffer "*Data List*" nil) (advance-direction)
      (if (file-readable-p (concat permas-doc-dir "dms_intern.chi"))
          (insert-file-contents (concat permas-doc-dir "dms_intern.chi"))
          (insert-file-contents (concat permas-doc-dir "dms.chi")))
      (set-buffer-modified-p nil) (setq buffer-read-only t)))
;;
(defun search-data-list (name)
   "Search for NAME in *Data List* and display help."
   (let ((actual-buffer (current-buffer)))
        (list-of-data) (set-buffer "*Data List*") (goto-char (point-min))
        (if (re-search-forward (concat "^" name "[ <]") nil t)
            (progn (show-help-for-data)
                   (end-of-line)
                   (if (re-search-forward (concat "^" name "[ <]") nil t)
                       (progn (beep)
                 (message (concat ">>> WARNING: Multiple " (upcase name)
                          " definitions => See Buffer *Data List* ! <<<")))))
            (beep) (message (concat "No such Function/Subroutine \""
                                     name "\" (search failed)")))
        (set-buffer actual-buffer)))

(defun show-help-for-data () ""
  (let*((NAME (upcase name))
        (line (buffer-linestring))
        ;;
        (data  (or (chi-get-field line) "???"))
        (ctx   (or (chi-get-field line "Ctx") "PERMAS"))
        (doc   (or (chi-get-field line "Doc") "???:??(??)"))
        (title (or (chi-get-field line "Tit") "No title-string !"))
        (type  (or (chi-get-field line "Typ") "Unknown Type"))
        (indent (+ 11 (length NAME))))
;;
  (if (not (string-match "[!:]$" title)) (setq title (concat title ":")))
  (with-output-to-temp-buffer "*Help*"
      (princ (format "%-63s%s\n%s\n%s\n" title ctx
                     (make-string (length title) ?-)
                     (chi-see-page doc)))
      (princ (format "%s   Type: %s\n%s" data type
                         (make-string (length NAME) ?=)))
      (print-help-return-message))))

;;----------------------------------------------------------------------------
;;;###autoload
(defun help-aba-keyword (&optional query)
"Show help for ABAQUS keyword of actual line (info from abadoor documentation).
Ask interactively for keyword if QUERY is non-nil (short cut otherwise)."
 (interactive)
 (call-help-with-default 'help-aba-key 'aba-keystring-at-point
                          query "UCI command"))
;;
(defun help-aba-key (keystring) "Display help for Abaqus KEYword."
; (setq test keystring);;;;;
  (let ((context (keystring-context keystring))
        (key     (keystring-key keystring))
        (subkey1 (keystring-subkey1 keystring))
        (subkey2 (keystring-subkey2 keystring))) ;; param1 param2 ????
  (let ((original-buffer (current-buffer)) (name "*abadoor-chi*")
        (KEY (if (string-match "^[$]" key) (substring key 1) key)) ; cut $key
        (TYP "ABA"))
       (abadoor-chi) (set-buffer name) (goto-char (point-min))
       (message KEY) (sleep-for 1)
       (if (or (re-search-forward (format "^%s %s %s.*@Ctx>%s;%s" KEY subkey1 subkey2 TYP context) nil t)
               (re-search-forward (format "^%s %s %s.*@Ctx>%s;"   KEY subkey1 subkey2 TYP) nil t)
               (re-search-forward (format "^%s %s.*@Ctx>%s;%s"    KEY subkey1 TYP context) nil t)
               (re-search-forward (format "^%s %s.*@Ctx>%s;"      KEY subkey1 TYP) nil t)
               (re-search-forward (format "^%s.*@Ctx>%s;%s"       KEY TYP context) nil t)
               (re-search-forward (format "^%s.*@Ctx>%s"          KEY TYP) nil t)
               (re-search-forward (format " %s %s.*@Ctx>%s;%s"    KEY subkey1 TYP context) nil t)
               (re-search-forward (format " %s %s.*@Ctx>%s"       KEY subkey1 TYP) nil t)
               (re-search-forward (format " %s.*@Ctx>%s"          KEY TYP) nil t)
               (re-search-forward (format "^%s .*@Ctx>"           KEY) nil t)
               (re-search-forward (format " %s .*@Ctx>"           KEY) nil t)
               (re-search-forward (format "%s.*@Ctx>"             KEY) nil t))
           (chi-view-page)
           (beep) (message (format
                  "No such %s-Command \"%s\" (search failed)" TYP KEY)))
       (set-buffer original-buffer))))
;;
(defun abadoor-chi () "Load context-help-index for abadoor to *abadoor-chi* buffer."
  (let ((name "*abadoor-chi*"))
       (if (get-buffer name) (set-buffer name)
           (unconditional-new-buffer name nil)
           (insert-file (permas-doc-file "abadoor" ".chi"))
           (set-buffer-modified-p nil) (setq buffer-read-only t))))

(defun aba-keystring-at-point () "Returns the keystring of actual Abaqus statement
in form \"KEY SUBKEY1 SUBKEY2\"."
  (condition-case () (save-excursion
  (let ((pos1)(pos2)(case-fold-search t)) ; Back to most-last *keyword
       (end-of-line) (re-search-backward "^ *[*] *[a-z]" nil t)
       (skip-chars-forward " *") (setq pos1 (point))
       (if (looking-at "[a-z]")
           (progn (skip-chars-forward "0-9A-Za-z ") (setq pos2 (point))
                  (if (< pos1 pos2)
                      (let ((key     (upcase (remove-blanks (buffer-substring-no-properties pos1 pos2))))
                            (subkey  (subkeys-at-point)))
                            (concat key (if subkey (concat " " subkey))))
                      (eval "[????]")))
           "[???]")))))

(defun remove-blanks (string) "Remove blanks from string."
      (replace-regexp-in-string " " "" string))

;;----------------------------------------------------------------------------
;;;###autoload
(defun help-rsc-variable (&optional query)
"Show help for PERMAS RSC variable of actual line (info from unix_install documentation).
Ask interactively for varibale name if QUERY is non-nil (short cut otherwise)."
 (interactive)
 (call-help-with-default 'help-um909-key 'keyword-at-point
                          query "RSC variable"))
;;
(defun help-um909-key (keystring) "Display help for um909/910 RSC variable."
  (let ((KEY (keystring-key keystring)))
  (let ((original-buffer (current-buffer)) (name "*um909-chi*"))
       (um909-chi) (set-buffer name) (goto-char (point-min))
       (message (concat "PERMAS Resource key " KEY))
       (if (or (re-search-forward (format "^%s .*@Ctx>"  KEY) nil t)
               (re-search-forward (format "%s.*@Ctx>"    KEY) nil t))
           (chi-view-page)
           (beep) (message (format
                  "No such RSC-Variable \"%s\" (search failed)" KEY)))
       (set-buffer original-buffer))))
;;
(defun um909-chi () "Load context-help-index for permas um909/910 to *um909-chi* buffer."
  (let ((name "*um909-chi*"))
       (if (get-buffer name) (set-buffer name)
           (unconditional-new-buffer name nil)
           (insert-file (permas-doc-file "unix_install" ".chi"))
           (set-buffer-modified-p nil) (setq buffer-read-only t))))

;;----------------------------------------------------------------------------
(provide 'gold_help)
