;;; This file is an addition to the GNU Emacs. GNU Emacs and the herein GOLD
;;; additions are distributed in the hope that it will be useful, but without
;;; ANY WARRANTY.  No author or distributor accepts responsibility to anyone
;;; for the consequences of using it or for whether it serves any particular
;;; purpose or works at all, unless he says so in writing.
;;; Everyone is granted permission to copy, modify and redistribute GNU Emacs,
;;; but only under the conditions described in the GNU Emacs General Public Li-
;;; cense.   A copy of this license is supposed to have been given to you along
;;; with GNU Emacs so you can know your rights and responsibilities.  It should
;;; be in a file named COPYING.  Among other things, the copyright notice and
;;; this notice must be preserved on all copies.
;;; Refer to the GNU Emacs General Public License for full details.
;;;
;;;############################################################################
;;;###  Additional/different PRIMARY definitions (simple move&scroll ...) ! ###
;;;###  This file is included (direct- or auto-loaded) by the main entry    ###
;;;###  file >>> gold.el <<< => Refer to that file for more informations.   ###
;;;############################################################################
;;; Bug reports and suggestions are always welcome !
;;
(gold-debug "gold_simple ")
;;
(defun keep-temporary-goal-column ()
  "Return t if `last-command' should keep `temporary-goal-column'."
  (memq last-command '(next-line     next-line-h
                       previous-line previous-line-h
                       scroll-down-h scroll-up-h scroll-up-down
                       scroll-half-down scroll-half-up)))
;;
(defun set-temporary-goal-column ()
  "Set the `temporary-goal-column' to latest horizontal position."
  (or (keep-temporary-goal-column)
      (setq temporary-goal-column
            (if (and track-eol (eolp)) 9999 (current-column)))))
;;
(defun gold-goal-column () "Save temporary-goal-column"
  (or goal-column 
      (if (consp temporary-goal-column) (truncate (car temporary-goal-column)))
      temporary-goal-column 0))
;;
(defun vertical-glue ()
  "Return t if `last-command' forces a vertical `glue-cursor'."
  (memq last-command '(glue-cursor vertical-glue)))
;;
(defun horizontal-glue ()
  "Return t if `last-command' forces a horizontal `glue-cursor'."
  (memq last-command '(glue-cursor horizontal-glue)))
;;
(defun previous-line-h ()
"Just `previous-line' with some additions (e.g support for glue-cursor function).
Move cursor vertically up ARG lines.
If there is no character in the target line exactly over the current column,
the cursor is positioned after the character in that line which spans this
column, or at the end of the line if it is not long enough."
  (interactive)
  (if (and (equal major-mode 'shell-mode) (comint-after-pmark-p))
      (call-interactively 'comint-previous-input)
  (if (equal major-mode 'hexl-mode) (call-interactively 'hexl-previous-line)
      (if (bobp) (message "Beginning of Buffer")
          (if (vertical-glue)
              (let ((column (current-column)))
                   (next-line 1) (transpose-lines -1)
                   (if (string-lessp emacs-version "24.4") (previous-line 1)
                   (if (string-lessp emacs-version "25.1") (previous-line 2) (previous-line 1)))
                   (move-to-column column) (setq this-command 'vertical-glue))
              ; keep temporary-goal-column in next line-move-1 !
              (if (keep-temporary-goal-column)    (setq last-command 'previous-line))
              (call-interactively 'previous-line) (setq this-command 'previous-line))
))))
;;
(defun next-line-h ()
"Just `next-line' with some additions (e.g. support for glue-cursor function).
Move cursor vertically down ARG lines.
If there is no character in the target line exactly under the current column,
the cursor is positioned after the character in that line which spans this
column, or at the end of the line if it is not long enough."
  (interactive)
  (if (and (equal major-mode 'shell-mode) (comint-after-pmark-p))
      (call-interactively 'comint-next-input)
  (if (equal major-mode 'hexl-mode) (call-interactively 'hexl-next-line)
      (if (eobp) (message "End of Buffer")
          (if (vertical-glue)
              (let ((column (current-column)))
                   (next-line 1) (transpose-lines 1) (previous-line 1)
                   (move-to-column column) (setq this-command 'vertical-glue))
              ; keep temporary-goal-column in next line-move-1 !
              (if (keep-temporary-goal-column) (setq last-command 'next-line))
              (call-interactively 'next-line)  (setq this-command 'next-line))
))))
;;
(defun forward-char-h ()
"Just forward-char with some additionals (e.g. special support
for glue-cursor). Move point right ARG characters (left if ARG negative).
On reaching end of buffer, stop and signal error."
  (interactive)
  (if (equal major-mode 'hexl-mode) (call-interactively 'hexl-forward-char)
      (if (horizontal-glue)
          (progn (forward-word 0) (transpose-words 1)
                 (setq this-command 'horizontal-glue))
          (call-interactively 'forward-char))))
;;
(defun backward-char-h ()
"Just backward-char with some additionals (e.g. support
for glue-cursor). Move point left ARG characters (right if ARG negative).
On reaching beginning of buffer, stop and signal error."
  (interactive)
  (if (equal major-mode 'hexl-mode) (call-interactively 'hexl-backward-char)
      (if (horizontal-glue)
          (progn (forward-word 0) (transpose-words -1)
                 (setq this-command 'horizontal-glue))
         (call-interactively 'backward-char))))
;;
(defun scroll-up-h (num)
  "Scroll the display up a window-full (half temporary column if possible).
Accepts a prefix argument for the number of window-fulls to scroll."
 (interactive "p")
 (if (equal major-mode 'calendar-mode) (call-interactively 'scroll-calendar-left)
  (if (equal major-mode 'hexl-mode) (call-interactively 'hexl-scroll-up)
      (set-temporary-goal-column)
      (scroll-up (- (* (window-height) num) 3))
      (move-to-column (gold-goal-column)))))
;;
(defun scroll-down-h (num)
  "Scroll the display up a window-full (half temporary column if possible).
Accepts a prefix argument for the number of window-fulls to scroll."
 (interactive "p")
 (if (equal major-mode 'calendar-mode) (call-interactively 'scroll-calendar-right)
  (if (equal major-mode 'hexl-mode) (call-interactively 'hexl-scroll-down)
      (set-temporary-goal-column)
      (scroll-down (- (* (window-height) num) 3))
      (move-to-column (gold-goal-column)))))

;;
(defun move-to-beginning ()
"Just move-to-beginning with some additionals.
Move cursor to the beginning of buffer, but don't set the mark."
  (interactive) (goto-char (point-min)) (advance-direction)
  (if (equal major-mode 'hexl-mode)
      (call-interactively 'hexl-beginning-of-buffer)))
;;
(defun move-to-end ()
"Just move-to-end with some additionals.
Move cursor to the end of buffer, but don't set the mark."
  (interactive) (goto-char (point-max)) (backup-direction)
  (if (equal major-mode 'hexl-mode)
      (call-interactively 'hexl-end-of-buffer)))
;;
(defun goto-column (column)
"Jump to given COLUMN in current line.
Also extends line with trailing blanks and/or expand tabs, if necessary."
  (interactive "nGoto column: ")
  (move-to-column (max 0 (- column 1)) t))
;;
(defun beginning-of-context ()
  "Move backward to next beginning of context (defun, routine, section, ...)"
  (interactive) (call-interactively (key-binding "\e\C-a")))
;;
(defun end-of-context ()
  "Move forward to next end of context (defun, routine, section, ...)"
  (interactive) (call-interactively (key-binding "\e\C-e")))
;;
(defun mark&copy-context ()
  "Mark&copy current context (defun, routine, section, ...)"
  (interactive)
  (end-of-context) (mark-paste-normal)
  (beginning-of-context) (copy-to-paste))

(defun cut-context-to-paste ()
  "Cut current context to paste-buffer (defun, routine, section, ...)"
  (interactive)
  (end-of-context) (mark-paste-normal)
  (beginning-of-context) (cut-to-paste))

;;****************************************************************************
;;***** Primitives for ADVANCE/BACKUP minor-mode ! ***************************
;;****************************************************************************
(defvar advance t
       "GOLD move/search/replace direction ADVANCE (t) or BACKUP (nil)")
;;
(defun toggle-direction ()
      "Toggle-Switch between the two minor modes ADVANCE and BACKUP !"
      (interactive)
      (if advance (backup-direction) (advance-direction)))
;;
(defun advance-direction ()
      "Set GOLD Advance mode so keypad commands move forward."
      (interactive) (setq advance t)
      (setq edt-direction-string " vvv") (force-mode-line-update))
;;
(defun backup-direction ()
      "Set GOLD Backup mode so keypad commands move backward."
      (interactive) (setq advance nil)
      (setq edt-direction-string " ^^^") (force-mode-line-update))
;;
(defun scroll-left-right (num)
  "Scroll the display left/right a few columns depending on ADVANCE/BACKUP mode.
Accepts a prefix argument for the number of columns to scroll (default is 10)."
  (interactive "p")
  (if advance (scroll-left (* num 10))
              (scroll-right (* num 10))))

(defun scroll-up-down (num)
  "Scroll the display down/up a window-full depending on ADVANCE/BACKUP mode.
Accepts a prefix argument for the number of window-fulls to scroll."
  (interactive "p")
  (if (equal major-mode 'calendar-mode)
      (call-interactively (if advance 'scroll-calendar-right
                                      'scroll-calendar-left))
     (set-temporary-goal-column)
     (if advance
     (if (pos-visible-in-window-p (point-max))
         (progn (beep)
                (message "Bottom of buffer reached:  Switched to BACKUP mode")
                (backup-direction))
         (scroll-up (- (* (window-height) num) 3)))
     (if (pos-visible-in-window-p (point-min))
         (progn (beep)
                (message "Top of buffer reached:  Switched to ADVANCE mode")
                (advance-direction))
         (scroll-down (- (* (window-height) num) 3))))
      (move-to-column (gold-goal-column))))

;;-----------------------
(defun GOLD-buffer-menu (&optional arg) "Just an extension of 'buffer-menu':
   Make a menu of buffers so you can save, delete or select them.
With argument, show only buffers that are visiting files.
Type ? after invocation to get help on commands available.
Type q immediately to make the buffer menu go away and to restore
previous window configuration."
  (interactive "P")
  (switch-to-buffer-other-window (list-buffers-noselect arg)) (forward-line 2)
  (message
   "Commands: d, s, x, u; f, o, 1, 2, m, v; ~, %%; q to quit; ? for help."))
;;
(define-key Buffer-menu-mode-map " "        'Buffer-menu-this-window)
(define-key Buffer-menu-mode-map [kp-enter] 'Buffer-menu-switch-other-window)
(define-key Buffer-menu-mode-map [select]   'Buffer-menu-1-window)
;;
(defun Buffer-menu-quit () "Quit the buffer menu."
  (interactive) (bury-buffer (current-buffer))
  (if (one-window-p) (switch-to-buffer (other-buffer))
                     (delete-window)))
;;-----------------------------------------------------------------------------
(defun rename-file-with-message (old new) " "
  (rename-file old new) (beep)
  (message (format "Renaming %s into %s !" old new))
  (sleep-for 1) (message " "))
;;-----------------------------------------------------------------------------
(setq comment-padding 0); number of extra spaces (or string) for comment-region
;;-----------------------------------------------------------------------------
(defun uncomment-region (beg end &optional arg)
  "Uncomment each line in the region (see `comment-region' also ).
Numeric prefix arg ARG means delete that many comment characters."
  (interactive "r\nP")
  (if arg (comment-region beg end (- arg))
          (comment-region beg end -1)))
;;-----------------------------------------------------------------------------
(defun find-executable-file (command)
  "Get complete executable file name (or nil)"
  (let ((which (shell-command-to-string (concat "which " command))))
    (if (or (< (length which) 1) (string-match "ommand not found" which)) nil
        (setq which (substring which 0 -1)))
        (if (file-executable-p which) which nil)))
;;-----------------------------------------------------------------------------
(provide 'gold_simple)
