;;; This file is an addition to the GNU Emacs. GNU Emacs and the herein GOLD
;;; additions are distributed in the hope that it will be useful, but without
;;; ANY WARRANTY.  No author or distributor accepts responsibility to anyone
;;; for the consequences of using it or for whether it serves any particular
;;; purpose or works at all, unless he says so in writing.
;;; Everyone is granted permission to copy, modify and redistribute GNU Emacs,
;;; but only under the conditions described in the GNU Emacs General Public Li-
;;; cense.   A copy of this license is supposed to have been given to you along
;;; with GNU Emacs so you can know your rights and responsibilities.  It should
;;; be in a file named COPYING.  Among other things, the copyright notice and
;;; this notice must be preserved on all copies.
;;; Refer to the GNU Emacs General Public License for full details.
;;;
;;;############################################################################
;;;###  Additional/different tag functions for gold.el !                    ###
;;;###  This file is included (direct- or auto-loaded) by the main entry    ###
;;;###  file >>> gold.el <<< => Refer to that file for more informations.   ###
;;;############################################################################
;;; Bug reports and suggestions are always welcome !
;;
;; Avoid buggy fileloop implementation for etags-27
(if (= emacs-major-version 27) (require 'etags26) (require 'etags))
;;
(if (string-lessp emacs-version "21.0") ; overwrite incomplete etags-function
(defun tags-reset-tags-tables ()
  "Reset tags state to cancel effect of any previous \\[visit-tags-table] or \\[find-tag]."
  (interactive)
  ;; Clear out the markers we are throwing away.
  (let ((i 0))
    (while (< i find-tag-marker-ring-length)
      (if (aref (cddr tags-location-ring) i)
          (set-marker (aref (cddr tags-location-ring) i) nil))
      (if (aref (cddr find-tag-marker-ring) i)
          (set-marker (aref (cddr find-tag-marker-ring) i) nil))
      (setq i (1+ i))))
  (setq tags-file-name nil
        tags-location-ring (make-ring find-tag-marker-ring-length)
        find-tag-marker-ring (make-ring find-tag-marker-ring-length)
        tags-table-list nil
        tags-table-computed-list nil
        tags-table-computed-list-for nil
        tags-table-list-pointer nil
        tags-table-list-started-at nil
        tags-table-set-list nil))
)
;;
(gold-debug "gold_tags")
;;------------------------
(defvar permas-source-dir "/source/" "*Main directory for PERMAS source files.")
;;
(defun emacs-tags () "Set 'tags-file-name' to an Emacs-Lisp tags file."
  (interactive) (tags-reset-tags-tables) (setq tags-table-list (list
   "/home/ast/files/EMACS.tags")))
;;
(defun permnew-tags () "Set 'tags-file-name' to PERMNEW tags file."
  (interactive) (tags-reset-tags-tables) (setq tags-table-list (list
  (concat permas-source-dir "permnew/etc/PERMAS"))))
;;
(defun permnew-f90-tags () "Set 'tags-file-name' to PERMNEW-F90 tags file."
  (interactive) (tags-reset-tags-tables) (setq tags-table-list (list
  (concat permas-source-dir "permnew/etc/PERMAS90"))))
;;
(defun permas-tags () "Set 'tags-file-name' to PERMAS tags file."
  (interactive) (tags-reset-tags-tables) (setq tags-table-list (list
  (concat permas-source-dir "permas/etc/PERMAS"))))
;;
(defun permas-f90-tags () "Set 'tags-file-name' to PERMAS-F90 tags file."
  (interactive) (tags-reset-tags-tables) (setq tags-table-list (list
  (concat permas-source-dir "permas/etc/PERMAS90"))))
;;
(defun permas-v18-tags () "Set 'tags-file-name' to PERMAS tags file."
  (interactive) (tags-reset-tags-tables) (setq tags-table-list (list
  (concat permas-source-dir "permas/V18/etc/PERMAS"))))
;;
(defun permas-v17-tags () "Set 'tags-file-name' to PERMAS tags file."
  (interactive) (tags-reset-tags-tables) (setq tags-table-list (list
  (concat permas-source-dir "permas/V17/etc/PERMAS"))))
;;
(defun permas-v16-tags () "Set 'tags-file-name' to old PERMAS V16 tags file."
  (interactive) (tags-reset-tags-tables) (setq tags-table-list (list
  (concat permas-source-dir "permas/V16/etc/PERMAS"))))
;;
(defun permas-v15-tags () "Set 'tags-file-name' to old PERMAS V15 tags file."
  (interactive) (tags-reset-tags-tables) (setq tags-table-list (list
  (concat permas-source-dir "permas/V15/etc/PERMAS"))))
;;
(defun permas-v14-tags () "Set 'tags-file-name' to old PERMAS V14 tags file."
  (interactive) (tags-reset-tags-tables) (setq tags-table-list (list
  (concat permas-source-dir "permas/V14/etc/PERMAS"))))
;;
(defun permas-v13-tags () "Set 'tags-file-name' to old PERMAS V13 tags file."
  (interactive) (tags-reset-tags-tables) (setq tags-table-list (list
  (concat permas-source-dir "permas/V13/etc/PERMAS"))))
;;
(defun permas-v12-tags () "Set 'tags-file-name' to old PERMAS V12 tags file."
  (interactive) (tags-reset-tags-tables) (setq tags-table-list (list
  (concat permas-source-dir "permas/V12/etc/PERMAS"))))
;;
(defun permas-v11-tags () "Set 'tags-file-name' to old PERMAS V11 tags file."
  (interactive) (tags-reset-tags-tables) (setq tags-table-list (list
  (concat permas-source-dir "permas/V11/etc/PERMAS"))))
;;
;(defun vgm-tags () "Set 'tags-file-name' to VisPer-Interface-Maintenance."
;  (interactive) (tags-reset-tags-tables) (setq tags-table-list (list
;  (concat permas-source-dir "vgm/etc/VGM"))))
;;
;(defun vgmper-tags () "Set 'tags-file-name' to PERMAS->VisPer-Maintenance."
;  (interactive) (tags-reset-tags-tables) (setq tags-table-list (list
;  (concat permas-source-dir "vgm/etc/VGM")
;  (concat permas-source-dir "permas/etc/PERMAS"))))
;;
(defun visper-tags () "Set 'tags-file-name' to VisPer-Maintenance."
  (interactive) (tags-reset-tags-tables) (setq tags-table-list (list
  (concat permas-source-dir "visper/FILES/VisPER")
  (concat permas-source-dir "permas/etc/PERMAS"))))
;;
(defun visnew-tags () "Set 'tags-file-name' to VisPer-Development."
  (interactive) (tags-reset-tags-tables) (setq tags-table-list (list
  (concat permas-source-dir "visnew/FILES/VisPER"))))
;;
;; Set default TAGS table according to $DEFPROG environment variable
(let ((DEFPROG (GETENV "DEFPROG")))
     (cond ((equal DEFPROG "permas")      (permas-tags))
           ((equal DEFPROG "permnew")     (permnew-tags))
           ((equal DEFPROG "visper")      (vgmper-tags))
           ((equal DEFPROG "visnew")      (visnew-tags))
;          ((equal DEFPROG "vgm")         (vgm-tags))
           (t                             (permas-tags))))
;;
;; Reread a TAGS table without querying, if it has changed.
(setq tags-revert-without-query t)
;(setq tags-add-tables nil) ; Do not concatenate tags-files ?

(defvar tags-read-only t
   "A non-nil value forces tags functions to load their files read-only !")
;;
(defvar last-tag-search  "?" "Type of last processed tag-searching")
(defun tags-find-file-hook () "Dummy function for setting tags-read-only !"
      (setq buffer-read-only tags-read-only))
;;
(defun find-tag-h ()
"Do regular `find-tag{-other-window}' and set the last-tag-search
to \"find-tag\" so repeat-tag-search can be used afterwards."
  (interactive) (setq this-command nil)
  (if (or (eq last-command 'find-tag-h)
          (eq last-command 'tags-search-h)) (repeat-tag-search)
  (let ((find-file-hook (append find-file-hook (list 'tags-find-file-hook))))
       (if (call-interactively 'find-tag-other-window)
;;;??? (if (call-interactively 'find-tag-other-window-f7x)
           (setq this-command 'find-tag-h))
       (setq last-tag-search "find-tag"))))
;;
;; Special clone of find-tag-other-window to search f7x-tmpl-regexp-tags
(defun find-tag-other-window-f7x (tagname &optional next-p)
  "find-tag-other-window with special regexp."
  (declare (obsolete xref-find-definitions-other-window "25.1"))
  (interactive (find-tag-interactive "Find tag other window: "))
  (let* ((window-point (window-point))
         (tagname (replace-regexp-in-string "<T>" "D" tagname))
;?       (tagname (replace-regexp-in-string "<L>" "2" tagname))
;?       (tagname (replace-regexp-in-string "<T>" "[SDCZ]" tagname))
;?       (tagname (replace-regexp-in-string "<L>" "[1248]" tagname))
;?       (tagbuf (find-tag-noselect tagname nil t))
         (tagbuf (find-tag-noselect tagname))
         (tagpoint (progn (set-buffer tagbuf) (point))))
    (set-window-point (prog1 (selected-window)
                        (switch-to-buffer-other-window tagbuf)
                        (set-window-point (selected-window) tagpoint))
                      window-point)))
;;
(defun tags-search-h ()
"Do regular tags-search and set the last-tag-search
to \"tags-search\" so repeat-tag-search can be used afterwards."
  (interactive)
  (let ((find-file-hook (append find-file-hook (list 'tags-find-file-hook)))
        (err-regexp (errnum-at-point)))
       (if err-regexp (tags-search err-regexp)
                      (call-interactively 'tags-search))
       (setq this-command 'tags-search-h)
       (setq last-tag-search "search-tags")))
;;
(defun repeat-tag-search ()
   "Repeat last TAG Search forward (find-tag or search-tags)."
  (interactive)
  (let ((find-file-hook (append find-file-hook (list 'tags-find-file-hook))))
       (cond ((equal last-tag-search "find-tag")
              (message (concat "Repeated find-tag: " last-tag))
              (find-tag nil t) (message " ")) ;; Use same window again !
;?            (find-tag nil t t) (message " ")) ;; Use same window again !
             ((equal last-tag-search "search-tags")
              (message "Repeated tags-search !") ;;;!!!
              (call-interactively 'tags-loop-continue) (message " "))
             (t (message "No repeatable tag search defined !")))))
;;
;; Return a default tag to search for, based on the text at point.
(defun find-tag-default () (f7x-function-at-point))

; Extend tag-symbol-match-p by special #member# tag
(fset 'orig-tag-symbol-match-p (symbol-function 'tag-symbol-match-p))
;
(defun tag-symbol-match-p (tag)
  (or (orig-tag-symbol-match-p tag)
      (tag-member-match-p tag)))
;
(defun tag-member-match-p (tag)  ;; Test for #tag# member tag
  (and (eq (char-after (point)) ?#)
       (eq (char-after (- (point) (length tag) 1)) ?#)))

;;----------------------------------------------------------------------------
(provide 'gold_tags)
