;;; This file is an addition to the GNU Emacs. GNU Emacs and the herein GOLD
;;; additions are distributed in the hope that it will be useful, but without
;;; ANY WARRANTY.  No author or distributor accepts responsibility to anyone
;;; for the consequences of using it or for whether it serves any particular
;;; purpose or works at all, unless he says so in writing.
;;; Everyone is granted permission to copy, modify and redistribute GNU Emacs,
;;; but only under the conditions described in the GNU Emacs General Public Li-
;;; cense.   A copy of this license is supposed to have been given to you along
;;; with GNU Emacs so you can know your rights and responsibilities.  It should
;;; be in a file named COPYING.  Among other things, the copyright notice and
;;; this notice must be preserved on all copies.
;;; Refer to the GNU Emacs General Public License for full details.
;;;
;;;############################################################################
;;;###  GOLD terminal initializations (terminal dependent Key-Bindings...)  ###
;;;###  This file is included (direct- or auto-loaded) by the main entry    ###
;;;###  file >>> gold.el <<< => Refer to that file for more informations.   ###
;;;############################################################################
;;; Bug reports and suggestions are always welcome !
(gold-debug "gold_term")
;;
;; Test on X-Terminal and define some auxiliary functions:
;;---------------------------------------------------------
(defvar x-terminal (or (equal window-system 'x) (equal window-system "x"))
  "*non-nil if actual terminal is a X-Terminal (X11-Window system running).")

(defvar windows (or (equal window-system 'w32) (equal window-system "w32")
                    (equal system-type "windows-nt"))
  "*non-nil if actual terminal is a Windows-Terminal (Windows system running).")
;;
(if windows (setq w32-enable-num-lock nil))

(defun GETENV (VAR)
   "Get the environment variable VAR and force it to be a non-nil string."
   (interactive) (let ((string (getenv VAR)))
                      (if (and string (stringp string)) string "unknown")))

(defun X-GETENV (VAR &optional DEFAULT)
   "Return a special X-Terminal/Host type for the environment variable VAR."
   (cond ((not x-terminal)                     nil)
         ((or (not VAR) (not (stringp VAR)))  (or TERM DEFAULT))
         ((string-match "hpterm"         VAR) "hpterm")
         ((string-match "ibm\\|aix"      VAR) "aix")
         ((string-match "sgi\\|iris"     VAR) "sgi")
         ((string-match "sun\\|sparc"    VAR) "sun")
         ((string-match "hp"             VAR) "hpterm")
         ((string-match "windows"        VAR) "windows")
         ((string-match "OEMCOMPUTER"    VAR) "windows")
         ( t                                  (or TERM DEFAULT))))

(defun shell-command-to-string (command)
  "Call unix COMMAND, wait for it and return its output as a string."
  (get-buffer-create "*scratch-buffer*")
  (set-buffer        "*scratch-buffer*")
  (call-process shell-file-name nil t nil "-fc" command)
  (prog1 (buffer-string) (kill-buffer "*scratch-buffer*")))
;;
(defvar xmodmap-program "xmodmap"
  "*Command string for xmodmap (e.g. full path name if not in exec-path).")
(if x-terminal
   (let ((which (shell-command-to-string "which xmodmap")))
        (if (string-match "ommand not found" which)
            (if (file-executable-p    "/usr/bin/X11/xmodmap")
                (setq xmodmap-program "/usr/bin/X11/xmodmap")
                (if (file-executable-p    "/usr/bin/xmodmap")
                    (setq xmodmap-program "/usr/bin/xmodmap")
                    (message "Can not find xmodmap in Your search path !")
                    (beep) (sleep-for 1))
))))
;;
(defvar xmodmap-pm nil "Output of latest 'xmodmap-pm' call.")
(defun  xmodmap-pm ()
  "Get actual settings from unix' \"xmodmap -pm\" on variable 'xmodmap-pm'."
  (interactive) (setq xmodmap-pm
  (shell-command-to-string (format "%s -pm" xmodmap-program))))
;;
(defvar xmodmap-pk nil "Output of latest 'xmodmap-pk' call.")
(defun  xmodmap-pk ()
  "Get actual settings from unix' \"xmodmap -pk\" on variable 'xmodmap-pk'."
  (interactive) (setq xmodmap-pk
  (shell-command-to-string (format "%s -pk" xmodmap-program))))
;;
;; Set values for environment-like machine variables:
;;----------------------------------------------------
(defvar TERM       (GETENV "TERM")    "*Original terminal type at emacs start")
(defvar HOST       (GETENV "HOST")    "*Original host at emacs start.")
(defvar REMOTEHOST (GETENV "REMOTEHOST") "*Original remotehost at emacs start")
;;;
(defvar DISPLAY (GETENV "DISPLAY") "*Original display at emacs start.")
(defvar XTERM (X-GETENV  DISPLAY TERM)
              "*Type of active X-Terminal (nil if no X-Window).")
(defvar XHOST (X-GETENV  HOST TERM)
              "*Type of active X-Host (nil if no X-Window).")
;;
(if (boundp 'keyboard-type)
    (or keyboard-type (setq keyboard-type (or XTERM TERM)))
    (defvar keyboard-type (or XTERM TERM)
           "Keyboad type for local terminal (standard equals (X)TERM)."))
;;
(defvar keyboard-language "USA"
       "Keyboard language (USA, ENGLISH, GERMAN, SWEDISH ...) !")


;; Do standard (Terminal-independent) Keybindings:
;;-------------------------------------------------
(require 'gold_keyboard)


;; Do X-Terminal-Stuff:
;;----------------------
(if x-terminal (load "gold_Xterm"   nil t))
(if windows    (load "gold_windows" nil t))


;; Specify some load functions to enable an interactive Re-Definition:
;;---------------------------------------------------------------------
(defun set-keyboard (term) "(Re)load KeyDefs for TERM !"
  (load "gold_keyboard")
  (if term (if (string= term "windows") (load "gold_windows")
               (let ((XTERM term)) (load "gold_Xterm")))))
;;
(defun set-keyboard-nonX    () "(Re)load KeyDefs for non-X-Terminal !"
                                  (interactive) (set-keyboard nil))
(defun set-keyboard-Xterm   () "(Re)load KeyDefs for (actual/standard) X-Terminal !"
                                  (interactive) (set-keyboard "Xterm"))
(defun set-keyboard-Xhpterm () "(Re)load KeyDefs for HP-like X-Terminal !"
                                  (interactive) (set-keyboard "Xhpterm"))
(defun set-keyboard-Xpc     () "(Re)load KeyDefs for PC-like X-Terminal !"
                                  (interactive) (set-keyboard "Xpc"))
(defun set-keyboard-Xsgi    () "(Re)load KeyDefs for SGI X-Terminal !"
                                  (interactive) (set-keyboard "Xsgi"))
(defun set-keyboard-Xsun    () "(Re)load KeyDefs for SUN X-Terminal !"
                                  (interactive) (set-keyboard "Xsun"))
(defun set-keyboard-windows () "(Re)load KeyDefs for Windows-Terminal !"
                                  (interactive) (set-keyboard "windows"))

;;----------------------------------------------------------------------------
(provide 'gold_term)
