;;; This file is an addition to the GNU Emacs. GNU Emacs and the herein GOLD
;;; additions are distributed in the hope that it will be useful, but without
;;; ANY WARRANTY.  No author or distributor accepts responsibility to anyone
;;; for the consequences of using it or for whether it serves any particular
;;; purpose or works at all, unless he says so in writing.
;;; Everyone is granted permission to copy, modify and redistribute GNU Emacs,
;;; but only under the conditions described in the GNU Emacs General Public Li-
;;; cense.   A copy of this license is supposed to have been given to you along
;;; with GNU Emacs so you can know your rights and responsibilities.  It should
;;; be in a file named COPYING.  Among other things, the copyright notice and
;;; this notice must be preserved on all copies.
;;; Refer to the GNU Emacs General Public License for full details.
;;;
;;;############################################################################
;;;###  Additional/different (La)TeX functions for gold.el !                ###
;;;###  This file is included (direct- or auto-loaded) by the main entry    ###
;;;###  file >>> gold.el <<< => Refer to that file for more informations.   ###
;;;############################################################################
;;; Bug reports and suggestions are always welcome !
;;
(gold-debug "gold_tex")
;;----------------------------------------------------------------------------
(add-hook 'tex-mode-hook        'TeX-mode-ON t)
(add-hook 'TeX-mode-hook        'TeX-mode-ON t)
(add-hook 'plain-tex-mode-hook  'TeX-mode-ON t)
(add-hook 'latex-mode-hook      'TeX-mode-ON t)
(add-hook 'LaTeX-mode-hook      'TeX-mode-ON t)
(defun TeX-mode-ON () "Additionals when changing to TeX-modes"
   (local-set-key "\"" 'self-insert-command) (alt-umlaut-on)
   (local-set-key "\t" 'tab-left-right)
   (if x-terminal (local-set-key "\n" 'delete-to-eol)  ; Strange Apollo-Keypad!
                  (local-set-key "\n" 'scroll-up-down)); Qume-"LFD"
   (if (featurep 'tex-site) (load "gold_auctex" nil t))
   ; Suppress silly proportional fonts in auc-tex
)
;; GOLD default is to *not* load buggy auctex
(if (featurep 'tex-site) (unload-feature 'tex-site))
;;
(defun intes-sty () "View `intes.sty' file for latex[gf]." (interactive)
  (if (file-readable-p "/doc/files/tex/intes/intes.sty")
  (find-file-read-only "/doc/files/tex/intes/intes.sty")
  (find-file-read-only "/usr/share/texmf/tex/latex/intes/intes.sty")))
;;
(defun um450-sty () "View `um450.sty' file for latex[gf]." (interactive)
  (find-file-read-only "/doc/inputs/sty/um450.sty"))
;;
(defun um450-xrf () "View `um450.xrf' file for latex[gf]." (interactive)
  (find-file-read-only "/doc/inputs/xrf/um450_new.xrf"))
;;
(defun um550-xrf () "View `um550.xrf' file for latex[gf]." (interactive)
  (find-file-read-only "/doc/inputs/xrf/um550_new.xrf"))
;;
(defun dump-groups () "View `dump_groups.tex' file for latex[gf]." (interactive)
  (find-file-read-only "/doc/intern/callref/include/error/dump_groups.tex"))

;;----------------------------------------------------------------------------
(defvar tex-error-regexp "^ ?! [^ ]"
   "Regular expression for TeX/LaTeX error messages.")
;;
;; Some dummies for debian-error on loading PERMAS/slides.tex & dump_groups.tex
(defun LaTeX-add-environments (&rest environments) "dummy")
(defun LaTeX-section-level    (name)               "dummy")
(defun LaTeX-add-index-entries (key)               "dummy")
(setq LaTeX-auto-regexp-list nil)                ; "dummy")
;;----------------------------------------------------------------------------
(defun preview-buffer () "Preview actual TeX-File (if DVI-File exists) !"
      (interactive) (preview (buffer-file-name)))
(defun preview (file) "Preview an existing DVI-File !"
      (interactive "fExisting DVI-File: ")
  (setq file (substring file 0 -4))
  (let ((dvi-file (concat file ".dvi"))
        (tty-file (concat file ".tty")))
      (if (not (file-readable-p dvi-file))
          (message (concat "No such file " dvi-file " (run TeX first) !"))
      (unconditional-new-buffer "*Preview*" t)
      (set-132-columns)
      (if (file-newer-than-file-p tty-file dvi-file)
          (message (concat "Loading old ASCII file " tty-file " !"))
          (message (concat "Converting dvi-file " dvi-file " into ASCII-file "
                           tty-file ", please wait !"))
          (call-process "dvitty" nil t nil file))
      (insert-file-contents tty-file) (not-modified) (setq buffer-read-only t)
)))

;;****************************************************************************
;;*****  Fax and Letter utilities  *******************************************
;;****************************************************************************
(defvar user-fax-dir   "~/tex/fax/" "Default directory for user's fax files !")
(defvar user-letter-dir "~/tex/letter/" "Default directory for letter files !")
(defun fax () "Load a TeX-file with intes own fax-dummy." (interactive)
   (Fax (read-file-name "Name of Fax-TeX-File (default fax.tex): "
                         user-fax-dir (concat user-fax-dir "fax.tex"))))
(defun Fax (file) "Load a TeX-file with intes own fax-dummy." (interactive)
   (let ((exist (file-exists-p file))) (find-file file)
        (if (not exist) (progn
            (insert "\\documentclass[12pt]{article}\n")
            (insert "\\usepackage{intes}\n")
            (insert "\\usepackage{fax}\n")
            (insert "\\DINA4\n\\begin{document}\n\\begin{fax}\n")
            (insert "%\n% Kopf\n%\n\\To{ }\n\\Att{ }\n\\Fax{ }\n\\From{")
            (insert user-extern-name)
            (insert "}\n\\Date{\\heute}\n\\Subject{ }\n\\Pages{ }\n")
            (insert "%\n% Inhalt\n%\n")
            (insert "\\Text{ }\n%\n\\end{fax}\n\\end{document}\n")))
        (move-to-beginning) (search-forward "\\To{")
))
(defun letter () "Load a TeX-file with intes own letter dummy."
   (interactive)
   (let*((file (read-file-name "Name of LetterTeX-File (default letter.tex): "
                     user-letter-dir (concat user-letter-dir "letter.tex")))
         (exist (file-exists-p file)))
        (find-file file)
        (if (not exist) (progn
            (insert "\\input{myletter}    % Initialize my letters\n")
            (insert "\\begin{document}    % Start document\n")
            (insert "%---------------------\n")
            (insert "\\INTESletter{ \\\\\n")
            (insert  "             Herrn , Abt. \\\\[1ex]\n")
            (insert  "             Postfach \\\\[1ex]\n")
            (insert  "      {\\bf D-}}\n")
            (insert "%\n\\Bezug{ }\n")
            (insert "%\n\\Betreff{ }\n\n")
            (insert "Sehr geehrter Herr ,\n\n\n")
            (insert "\\Gruss\n\n")
            (insert "%\\Anlage{DAT-Tape}\n")
            (insert "%\\B{Anlagen}\n")
            (insert "%   2 & PERMAS User's Reference Manual   \\\\\n")
            (insert "%\\E{Anlagen}\n")
            (insert "%\\Aufkleber\n")
            (insert "%---------------------\n")
            (insert "\\end{document}\n")))
        (move-to-beginning) (search-forward "\\INTESletter{")
))
;;----------------------------------------------------------------------------
(defvar TeX-master-default "main" "*Default file name for `TeX-master'.")
(defun insert-tex-trailer () "Insert Auc-TeX trailer for file-local variables."
   (interactive) (goto-char (point-max))
   (insert "
%%%----------------------------------------------------------------------------
%%% Local " "Variables:
%%% mode: LaTeX
%%% TeX-auto-save: t
%%% TeX-parse-self: t
%%% TeX-master: \"" TeX-master-default "\"
%%% End:
%%%----------------------------------------------------------------------------
") (search-backward TeX-master-default nil t))
;;
(eval-after-load "tex-buf"  ;; Correcting tex-buf error !
  '(defun TeX-region-file (&optional extension)
     "Return TeX-region file name with EXTENSION (gold-version)."
     (let ((name (concat (TeX-master-file) TeX-region)))
          (cond ((eq extension t) (concat name "." TeX-default-extension))
                (extension (concat name "." extension))
                (t name))))
)
;;----------------------------------------------------------------------------
(provide 'gold_tex)
