;;; This file is an addition to the GNU Emacs. GNU Emacs and the herein GOLD
;;; additions are distributed in the hope that it will be useful, but without
;;; ANY WARRANTY.  No author or distributor accepts responsibility to anyone
;;; for the consequences of using it or for whether it serves any particular
;;; purpose or works at all, unless he says so in writing.
;;; Everyone is granted permission to copy, modify and redistribute GNU Emacs,
;;; but only under the conditions described in the GNU Emacs General Public Li-
;;; cense.   A copy of this license is supposed to have been given to you along
;;; with GNU Emacs so you can know your rights and responsibilities.  It should
;;; be in a file named COPYING.  Among other things, the copyright notice and
;;; this notice must be preserved on all copies.
;;; Refer to the GNU Emacs General Public License for full details.
;;;
;;;############################################################################
;;;###  PostScript-View facilities (context-help and others) for gold.el !  ###
;;;###  This file is included (direct- or auto-loaded) by the main entry    ###
;;;###  file >>> gold.el <<< => Refer to that file for more informations.   ###
;;;############################################################################
;;; Bug reports and suggestions are always welcome !
;;
(gold-debug "gold_view")
;;
;;;---------------------------------------------------------------------------
(defvar pdfview-program nil "*Program to view PDF files (nil if none).")
(if (not pdfview-program)
    (if windows (setq pdfview-program "acroread")
                (setq pdfview-program (or (find-executable-file "xpdf")
                                          (find-executable-file "qpdfview")
                                          (find-executable-file "evince")
                                          (find-executable-file "okular")
                                          (find-executable-file "acroread")))))
;;;---------------------------------------------------------------------------
;;
(defvar xpdf-version "4.0" "xpdf version")
(defun get-xpdf-version () "Get xpdf version from shell"
  (interactive) (setq xpdf-version (shell-command-to-string 
  "xpdf -v |& grep version | sed -e 's/[a-z ]//g'")))
(get-xpdf-version)
;
(defun xpdf (pdffile abspage title) ""
  (start-process "xpdf_server" nil shell-file-name "-fc" (format
           (if (string-lessp xpdf-version "4.0") 
               "xpdf -remote emacs %s %s" ;3.x
               "xpdf -remote emacs 'checkOpenFileAtPage(%s,%s)'") pdffile abspage))
  (process-kill-without-query (get-process "xpdf_server")))
;;
(defun qpdfview (pdffile abspage title) ""
  (start-process "qpdfview_server" nil shell-file-name "-fc" (format
                 "qpdfview --unique %s#%s" pdffile abspage))
  (process-kill-without-query (get-process "qpdfview_server")))
;;
(defun evince (pdffile abspage title) ""
  (start-process "evince_server" nil shell-file-name "-fc" (format
                "evince -p %s %s" abspage pdffile))
  (process-kill-without-query (get-process "evince_server")))
;;
(defun okular (pdffile abspage title) ""
  (start-process "okular_server" nil shell-file-name "-fc" (format
                "okular -p %s --unique %s" abspage pdffile))
  (process-kill-without-query (get-process "okular_server")))
;;
(defun acroread (pdffile abspage title) ""
  (if windows
     (start-process-shell-command "acroread_server" nil
                          (format "pdfc.exe -p %s \"%s\"" abspage pdffile))
     (start-process "acroread_server" nil shell-file-name "-fc"
            (format "acroread %s" pdffile)))
  (process-kill-without-query (get-process "acroread_server")))
;;;---------------------------------------------------------------------------
;;
;;;---
;;(setq show-help-function 'tooltip-show-help-function)
(defun uci-tooltip () "Display help for UCI KEYword."
  (let ((key (keyword-at-point)))
  (save-excursion
       (um450-chi) (goto-char (point-min))
       (if (re-search-forward (format "^%s " key) nil t)
           (chi-title)))))
;(put-text-property (point-min) 50 'help-echo
; (lambda (window object point) (uci-tooltip)
;))
;;
;;;---------------------------------------------------------------------------
(provide 'gold_view)
