;;; permas_pqa.el --- PERMAS PQA mode (viewing PERMAS PQA log files)

;; Copyright (C) 1999-2015 Free Software Foundation, Inc.

;; Last Change: 20-Oct-22
;; Keywords: PERMAS, PQA, data, browsing

;; This file is part of GNU Emacs.

;; GNU Emacs is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2, or (at your option)
;; any later version.

;; GNU Emacs is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with GNU Emacs; see the file COPYING.  If not, write to the
;; Free Software Foundation, Inc., 59 Temple Place - Suite 330,
;; Boston, MA 02111-1307, USA.

;;; Commentary:

;; permas-pqa-mode is a major mode for viewing PERMAS PQA log files.

;;; Code:

;;;###autoload
(defun permas-pqa-mode () "Viewing mode for PERMAS PQA log files."
  (interactive) (text-mode) ; start with text-mode initialization
  (setq mode-name  "PERMAS-PQA")
  (setq major-mode 'permas-pqa-mode)
  (remap-function-keys); Remap [kp-f1] to [gold] if StartUp with PQA
  (make-pqa-mode-map) (use-local-map permas-pqa-mode-map)
; (setq context-help-function   'permas-pqa-help)
; (setq context-update-function 'permas-pqa-context-update)
  (set (make-local-variable 'font-lock-defaults) '(permas-pqa-font-lock-keywords t t))
  (if font-lock-mode (font-lock-mode 1)) ;no guess why we need this
  (run-mode-hooks 'permas-pqa-mode-hook))

;;
;; font-lock support
;;-------------------
(setq permas-pqa-font-lock-keywords
  (list
   '("^><><> [^ ]*" . 'font-lock-separator-face)
   '("^>>>>> .*"    . 'font-lock-warning-face)
   '("^>>>>  OK .*" . 'font-lock-type-face)
   '("^>>.*"        . 'font-lock-string-face)
))

;;
;; Create local PQA-Keymap
;;-------------------------
(defvar permas-pqa-mode-map nil "Local keymap used in permas-pqa-mode.")
(defun make-pqa-mode-map ()
(if  permas-pqa-mode-map () (setq permas-pqa-mode-map (make-sparse-keymap))
  (define-key permas-pqa-mode-map "\e\C-a"   'permas-pqa-prev)
  (define-key permas-pqa-mode-map "\e\C-e"   'permas-pqa-next)
  (define-key permas-pqa-mode-map "<"        'permas-pqa-prev)
  (define-key permas-pqa-mode-map ">"        'permas-pqa-next)
  (define-key permas-pqa-mode-map ","        'permas-pqa-prev)
  (define-key permas-pqa-mode-map "."        'permas-pqa-next)
  (define-key permas-pqa-mode-map ""       'permas-pqa-next)
  (define-key permas-pqa-mode-map " "        'permas-pqa-next)
;;
  (define-key permas-pqa-mode-map "a"        'permas-pqa-get-aba)
  (define-key permas-pqa-mode-map "c"        'permas-pqa-plot-conv)
  (define-key permas-pqa-mode-map "d"        'permas-pqa-get-dat)
  (define-key permas-pqa-mode-map "D"        'permas-pqa-get-diff)
  (define-key permas-pqa-mode-map "f"        'permas-pqa-get-res-file)
  (define-key permas-pqa-mode-map "F"        'permas-pqa-get-source-file)
  (define-key permas-pqa-mode-map "h"        'permas-pqa-get-help-file)
  (define-key permas-pqa-mode-map "n"        'permas-pqa-get-nas)
  (define-key permas-pqa-mode-map "u"        'permas-pqa-get-uci)
  (define-key permas-pqa-mode-map "i"        'permas-pqa-get-unv)
  (define-key permas-pqa-mode-map "l"        'permas-pqa-get-log-both)
  (define-key permas-pqa-mode-map "L"        'permas-pqa-get-log-new)
  (define-key permas-pqa-mode-map "p"        'permas-pqa-get-pro-both)
  (define-key permas-pqa-mode-map "P"        'permas-pqa-get-pro-new)
  (define-key permas-pqa-mode-map "r"        'permas-pqa-get-res-both)
  (define-key permas-pqa-mode-map "R"        'permas-pqa-get-res-new)
;
; (define-key permas-pqa-mode-map [home]     'permas-pqa-export-ok)
; (define-key permas-pqa-mode-map [end]      'permas-pqa-export-not-ok)
; (define-key permas-pqa-mode-map [kp-enter] 'permas-pqa-export-skip)
; (define-key permas-pqa-mode-map [kp-add]   'permas-pqa-reload)
;;
  (substitute-key-definition 'show-next-error     'permas-pqa-next-error
                              permas-pqa-mode-map  global-map)
  (substitute-key-definition 'comment-region      'permas-pqa-next-export-error
                              permas-pqa-mode-map  global-map)
  (substitute-key-definition 'overwrite-mode      'permas-pqa-ok
                              permas-pqa-mode-map  global-map)
  (substitute-key-definition 'delete-current-char 'permas-pqa-not-ok
                              permas-pqa-mode-map  global-map)
  (substitute-key-definition 'newline             'permas-pqa-skip
                              permas-pqa-mode-map  global-map)
))

;; Next/prev PQA-Header
;;----------------------
(defun permas-pqa-prev (&optional arg nocenter)
"Move to start of current PQA log. With argument ARG, do it that many times.
Negative ARG means move forward to Nth following start of data block.
If NOCENTER is nil (default) make header to top-of-window."
  (interactive "p") (or (bolp) (end-of-line 1))
;;  (and (re-search-backward "^><><> " nil 'move (or arg 1))
  (previous-line 2)
  (and (re-search-backward
"^-------------------------------------------------------------------------------$"
        nil 'move (or arg 1))
       (progn (next-line 1) (beginning-of-line)
              (or nocenter
              (if (eobp) (line-to-bottom-of-window) (line-to-top-of-window) t)))))
;;
(defun permas-pqa-next (&optional arg nocenter)
  "Move forward to next PQA log. With argument ARG, do it that many times.
Negative ARG means move back to Nth preceding end of xydata.
If NOCENTER is nil (default) make header to top-of-window."
  (interactive "p") (if (or (null arg) (= arg 0)) (setq arg 1))
  (forward-line 2) (permas-pqa-prev (- arg) nocenter))

;;
;; Next error
::------------
(defun permas-pqa-next-error () "Move forward to next PQA error."
  (interactive)
  (permas-pqa-next 1 t)
  (if (re-search-forward "^>>>>> " nil 'move)
      (progn (save-window-excursion (permas-pqa-prev 1 t))
             (if (re-search-forward "^>>>>" nil 'move)
                 (progn (previous-line 3)  (line-to-top-of-window))))))
;;
(defun permas-pqa-next-export-error () "Move forward to next PQA export error."
  (interactive)
  (permas-pqa-next 1 t)
  (if (re-search-forward "^>>>>  Differences " nil 'move)
      (if (re-search-backward "^>> Export "  nil 'move)
          (line-to-top-of-window))))
;;
(defun permas-pqa-reload () "Reload PQA directory list."
  (interactive)
  (other-window 1)
  (insert "r")
  (comint-send-input)
  (other-window -1))
;;
(defun permas-pqa-ok () "*OK* PQA differences, update reference files."
  (interactive)
  (other-window 1)
  (insert "y ALL")
  (comint-send-input)
  (other-window -1)
  (permas-pqa-next-error))
;;
(defun permas-pqa-not-ok () "*BAD* PQA differences, delete output files."
  (interactive)
  (other-window 1)
  (insert "d ALL")
  (comint-send-input)
  (other-window -1)
  (permas-pqa-next-error))
;;
(defun permas-pqa-skip () "*SKIP* PQA differences, leave output files untouched."
  (interactive)
  (other-window 1)
  (insert "n")
  (comint-send-input)
  (other-window -1)
  (permas-pqa-next-error))
;;
(defun permas-pqa-export-ok () "*OK* PQA differences, update reference files."
  (interactive)
  (other-window 1)
  (insert "y ALL")
  (comint-send-input)
  (other-window -1)
  (permas-pqa-next-export-error))
;;
(defun permas-pqa-export-not-ok () "*BAD* PQA differences, delete output files."
  (interactive)
  (other-window 1)
  (insert "d ALL")
  (comint-send-input)
  (other-window -1)
  (permas-pqa-next-export-error))
;;
(defun permas-pqa-export-skip () "*SKIP* PQA differences, leave output files untouched."
  (interactive)
  (other-window 1)
  (insert "n")
  (comint-send-input)
  (other-window -1)
  (permas-pqa-next-export-error))

;;
;; Load related files
;;--------------------
(defun permas-pqa-get-uci () "Load uci file for current PQA"
   (interactive) (permas-pqa-get-file "Source" "uci"))
;;
(defun permas-pqa-get-aba () "Load Abaqus file for current PQA"
   (interactive) (permas-pqa-get-file "Source" "inp"))
;;
(defun permas-pqa-get-dat () "Load dat file for current PQA"
   (interactive) (permas-pqa-get-file "Source" "dat"))
;;
(defun permas-pqa-get-diff () "Load diff file for current PQA"
   (interactive) (permas-pqa-get-file "Result" "diff"))
;;
(defun permas-pqa-get-res-file () "Interactively load res-file for current PQA"
   (interactive) (permas-pqa-get-file "Result" nil))
;;
(defun permas-pqa-get-source-file () "Interactively load source-file for current PQA"
   (interactive) (permas-pqa-get-file "Source" nil))
;;
(defun permas-pqa-get-nas () "Load nas file for current PQA"
   (interactive) (permas-pqa-get-file "Source" "nas"))
;;
(defun permas-pqa-get-unv () "Load unv file for current PQA"
   (interactive) (permas-pqa-get-file "Source" "unv"))
;;
(defun permas-pqa-get-help-file () "Load help file for current PQA"
   (interactive) (permas-pqa-get-file "Source" ".pqa_remark"))
;;
(defun permas-pqa-get-pro-old () "Load pro file for current PQA"
   (interactive) (permas-pqa-get-file "Result" "pro"))
;;
(defun permas-pqa-get-pro-new () "Load new pro file for current PQA"
   (interactive) (permas-pqa-get-file "Result" "pro.new"))
;;
(defun permas-pqa-get-pro-both () "Load pro & pro.new file for current PQA"
   (interactive) (permas-pqa-get-file "Result" "pro" "pro.new"))
;;
;;
(defun permas-pqa-get-res-old () "Load res file for current PQA"
   (interactive) (permas-pqa-get-file "Result" "res"))
;;
(defun permas-pqa-get-res-new () "Load new res file for current PQA"
   (interactive) (permas-pqa-get-file "Result" "res.new"))
;;
(defun permas-pqa-get-res-both () "Load res & res.new file for current PQA"
   (interactive) (permas-pqa-get-file "Result" "res" "res.new"))
;;
;;
(defun permas-pqa-get-log-old () "Load log file for current PQA"
   (interactive) (permas-pqa-get-file "Result" "log"))
;;
(defun permas-pqa-get-log-new () "Load new log file for current PQA"
   (interactive) (permas-pqa-get-file "Result" "log.new"))
;;
(defun permas-pqa-get-log-both () "Load log & log.new file for current PQA"
   (interactive) (permas-pqa-get-file "Result" "log" "log.new"))
;;
(defun permas-pqa-get-file (dirname extension &optional alternative)
   "Load a file from results/source-directory for current PQA"
   (let ((project) (path) (base) (file))
        (save-excursion
          (next-line 2)
          (permas-pqa-prev 1 t)
          (re-search-forward "^><><> *\\([^ ]*\\) ")
          (setq project (match-string 1))
          (re-search-forward (concat "^" dirname
                             " reference directory: *\\([^ ]*\\) *$"))
          (setq path (match-string 1))

          ;; standard must match
          (setq file (concat path "/"))
          (if extension
              (if (string= extension ".pqa_remark")
                  (progn (setq file (concat path "/.pqa_remark")))
                  (progn (setq base (concat path "/" project "." extension))
                         (setq file base)
                         (or (file-exists-p file) (setq file (concat base ".gz")))
                         (or (file-exists-p file) (setq file (concat base ".zst")))
              )))
          (if (file-exists-p file) (permas-pqa-get-filex file)
              (beep) (message (concat "No such file " file)))

          ;; first load alternative (if any)
          (if alternative (progn
              (setq base (concat path "/" project "." alternative))
              (setq file base)
              (or (file-exists-p file) (setq file (concat base ".gz")))
              (or (file-exists-p file) (setq file (concat base ".zst")))
              (if (file-exists-p file) (permas-pqa-get-filex file))))
)))

;;
(defun permas-pqa-get-filex (file) "Load a PERMAS-PQA file"
   (find-file-other-window file)
   (setq buffer-read-only t))

;; CWE's log-file test
(defun plot-conv (logfile1 logfile2)
  (start-process "plot_conv" nil shell-file-name "-fc" (format
                 "plot_conv_crit.sh %s %s" logfile1 logfile2))
  (process-kill-without-query (get-process "plot_conv")))
;;
(defun permas-pqa-plot-conv ()
   "Load both log files and call plot-conv process"
   (interactive)
   (let ((project) (path) (base) (file1) (file2))
        (save-excursion
          (next-line 2)
          (permas-pqa-prev 1 t)
          (re-search-forward "^><><> *\\([^ ]*\\) ")
          (setq project (match-string 1))
          (re-search-forward "^Result reference directory: *\\([^ ]*\\) *$")
          (setq path (match-string 1))
          (setq base (concat path "/" project ".log"))
          (setq file1 base)
          (or (file-exists-p file1) (setq file1 (concat base ".gz")))
          (or (file-exists-p file1) (setq file1 (concat base ".zst")))
          (setq base (concat path "/" project ".log.new"))
          (setq file2 base)
          (or (file-exists-p file2) (setq file2 (concat base ".gz")))
          (or (file-exists-p file2) (setq file2 (concat base ".zst")))
          (if (file-exists-p file1)
              (if (file-exists-p file2) (plot-conv file1 file2)
                  (beep) (message (concat "No such file " file2)))
              (beep) (message (concat "No such file " file1)))
)))

;;
(defun permas-pqa-remove-directory () "*RMDIR* PQA directory, remove result directory "
  (interactive)
  (other-window 1)
  (insert "c")
  (comint-send-input)
  (other-window -1)
  (permas-pqa-next-error))

;;
(provide 'permas_pqa)

;;; permas_pqa.el ends here
