;;; permas_uci.el --- PERMAS UCI mode (editing PERMAS UCI files)

;; Copyright (C) 1999-2015 Free Software Foundation, Inc.

;; Last Change: 31-Jul-15
;; Keywords: PERMAS, UCI, editing

;; This file is part of GNU Emacs.

;; GNU Emacs is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2, or (at your option)
;; any later version.

;; GNU Emacs is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with GNU Emacs; see the file COPYING.  If not, write to the
;; Free Software Foundation, Inc., 59 Temple Place - Suite 330,
;; Boston, MA 02111-1307, USA.

;;; Commentary:

;; permas-uci-mode is a major mode for editing PERMAS UCI files.

;;; Code:

;;;###autoload
(defun permas-uci-mode ()
  "Text-mode for viewing/editing PERMAS UCI-Files."
  (interactive) (text-mode) (setq mode-name "PERMAS-UCI")
  (make-local-variable 'comment-start) (setq comment-start "!")
  (setq major-mode 'permas-uci-mode)
  (setq compile-buffer-function 'permas-run-buffer)
  (set (make-local-variable 'font-lock-defaults) '(permas-uci-font-lock-keywords t t))
  (substitute-key-definition 'split-line 'uci-split-line (current-local-map) global-map)
  (if font-lock-mode (font-lock-mode 1)) ;no guess why we need this
  (run-mode-hooks 'permas-uci-mode-hook))

;;;###autoload
(defun permas-stop-mode ()
  "Text-mode for viewing/editing PERMAS User-Stop-Files."
  (interactive) (text-mode) (setq mode-name "PERMAS-STOP-File")
  (make-local-variable 'comment-start) (setq comment-start "!")
  (setq major-mode 'permas-stop-mode)
  (set (make-local-variable 'font-lock-defaults) '(permas-uci-font-lock-keywords t t))
  (substitute-key-definition 'split-line 'uci-split-line (current-local-map) global-map)
  (if font-lock-mode (font-lock-mode 1)) ;no guess why we need this
  (run-mode-hooks 'permas-uci-mode-hook))
;;
(defun uci-split-line ()
  "Break line at point and insert continuation marker and alignment."
  (interactive) (delete-horizontal-space) (let (colpos)
  (if (save-excursion (re-search-backward "^\\|!")
                      (setq colpos (current-column)) (looking-at "!"))
      (insert (concat "\n" (make-string colpos ? ) "! "))
      (if (looking-at " *!") (insert "\n") (insert " &\n   ")))))


;; font-lock support (list of regular expressions for highlighting)
;;-----------------------------------------------------------------
(setq permas-uci-font-lock-keywords
  (list
   ;; TASK command (special keywords)
   '("task\\([:blank:]*\\(store\\|end\\|run\\)\\)*"
                                       . 'font-lock-match-face)
   ;;
   ;; SECTION (entry commands)
   '("\\(exec\\|export\\|input\\|license\\|new\\|print\\|select\\|startup\\|stop\\|user\\)"
                                       . 'font-lock-separator-face)
   ;;
   ;; REPORT keyword on port
   '("report[:blank:]+\\([^;:=!\n]+\\)[:blank:]+\\(on\\|off\\)" (1 'font-lock-type-face))
   ;;
   ;; DUMP 999 ON/OFF
   '("dump[:blank:]+\\([0-9]+\\)"             (1 'font-lock-separator-face))
   ;;
   ;; Special KEYWORDs
   '("\\(port\\|trap\\)" . 'default)
   ;;
   ;; KEYWORD subkeys (general keyword, 1st in line or 1st after ;)
   '("\\(^\\|;\\)[:blank:]*\\([a-z0-9_ \t]+\\)" (2 'font-lock-function-name-face))
   ;;
   ;; keyword = value
   '("=\\([^;:=!\n]*\\)"              (1 'font-lock-variable-name-face t))
   '("\\([a-z][^ \t]*\\) *="          (1 'font-lock-type-face t)) ; t=Overwrite
   ;;
   ;; Book names (with dot)
   '("\\b[a-z0-9]+\\.[a-z0-9]*\\b"    (0 'font-lock-variable-name-face t))
   ;;
   ;; continuation-fields (and following list of integers)
   '("[&][ \t0-9]*"                    . 'font-lock-string-face)
   ;;
   ;; data-field separator
   '("[:;]"                            . 'font-lock-separator-face)
   ;;
   ;; Standard comments
   '("!.*"                            (0 'font-lock-comment-face t))
))

;;
;;??
(defun permas-run-buffer () " "
  ;; kill optionally existing xxxx.log buffer visiting previous permas outputs
  (let* ((log-file (concat (file-name-sans-extension (buffer-file-name)) ".log"))
         (permas-buffer (get-file-buffer log-file)))
        (if permas-buffer (kill-buffer permas-buffer)))
  ;; Invoke compile command
  (setq compile-command "permnew")
  (setq jobname "ca1")
  (compile-internal (concat compile-command " " jobname) "No more errors")
  (message (concat "PERMAS run for " jobname " started !")))


(provide 'permas_uci)

;;; permas_uci.el ends here
