;;; visper_tip.el --- VisPer tooltip mode (editing VisPer Tooltip files)

;; Copyright (C) 2007 Free Software Foundation, Inc.

;; Last Change: 13-Feb-07
;; Keywords: VisPer, Tooltips, editing

;; This file is part of GNU Emacs.

;; GNU Emacs is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2, or (at your option)
;; any later version.

;; GNU Emacs is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with GNU Emacs; see the file COPYING.  If not, write to the
;; Free Software Foundation, Inc., 59 Temple Place - Suite 330,
;; Boston, MA 02111-1307, USA.

;;; Commentary:

;; visper-tip-mode is a major mode for editing VisPer Tip files.

;;; Code:

;;;###autoload
(defun visper-tip-mode ()
  "Editing mode for VisPer Tip files (*.tpl)."
  (interactive) (html-mode) ; start with html-mode initialization
  (setq mode-name  "VisPer-Tip")
  (setq major-mode 'visper-tip-mode)
;;
  (use-local-map visper-tip-mode-map)
  (local-set-key "\e\C-a"    'visper-tip-prev)
  (local-set-key "\e\C-e"    'visper-tip-next)
;;
  (make-local-variable 'comment-start) (setq comment-start ":")
  (set (make-local-variable 'font-lock-defaults) '(visper-tip-font-lock-keywords t t))
  (if font-lock-mode (font-lock-mode 1)) ;no guess why we need this
  (run-mode-hooks 'visper-tip-mode-hook))

(defvar visper-tip-mode-map (make-sparse-keymap)
   "Local keymap used in visper-tip-mode.")


;; font-lock support (list of regular expressions for highlighting)
;;-----------------------------------------------------------------
(setq visper-tip-font-lock-keywords
  (list
   ;; Header
   '("^#?:.*" . 'font-lock-comment-face)
   ;;
   ;; #object : key -> key
   '("^\\(#\\) *\\([^ :]*\\) *\\(:\\)\\(.*\\)->\\(.*\\)"
     (1 'font-lock-match-face)
     (2 'font-lock-reference-face)
;    (2 'font-lock-variable-name-face)
     (3 'font-lock-comment-face)
;    (4 'font-lock-type-face)
     (4 'font-lock-variable-name-face)
     (5 'font-lock-function-name-face))
   ;;
   '("</?[[:alnum:]][-_.:[:alnum:]]*>" . 'font-lock-type-face)
   ;;
   ;; data-field separator
   '("^-"                             . 'font-lock-separator-face)
   ;;
   ;; Standard comments
   '("^# .*"                          . 'font-lock-comment-face)
))
;(setq visper-tip-font-lock-keywords
;  (append visper-tip-font-lock-keywords-1 sgml-font-lock-keywords))



;; Next/prev header line
;;-----------------------
(defun visper-tip-next (&optional arg)
  "Move forward to next tooltip. With argument, do it that many times.
Negative argument -N means move back to Nth preceding end of xydata."
  (interactive "p")
  (or (bolp) (end-of-line 1))
  (if (re-search-forward "^#[^ ]" nil t (or arg 1)) (forward-line 1)))
;;
(defun visper-tip-prev (&optional arg)
"Move to start of current tooltip. With argument, do it that many times.
Negative arg -N means move forward to Nth following start of data block."
  (interactive "p")
  (or (bolp) (end-of-line 1))
  (re-search-backward "^#[^ ]" nil 'move (or arg 1)))

;;;----------------------------------------------------------------------------

(provide 'visper_tip)

;;; visper_dat.el ends here
